// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.dialogs.filteredSelection;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.Representation;
import eu.etaxonomy.cdm.model.common.TermVocabulary;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.common.VocabularyEnum;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.taxeditor.editor.newWizard.AbstractNewEntityWizard;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created May 11, 2010
 * @version 1.0
 */
public class FilteredNamedAreaSelectionDialog extends
		AbstractFilteredCdmResourceSelectionDialog<NamedArea> {

	private static final Logger logger = Logger
			.getLogger(FilteredNamedAreaSelectionDialog.class);

	private class IncludeTdwgAreaAction extends Action {
		/**
		 * Creates a new instance of the class.
		 */
		public IncludeTdwgAreaAction() {
			super("TDWG Areas", IAction.AS_CHECK_BOX);
		}

		public void run() {
			if(isChecked()){
				selectedVocabularies.add(VocabularyEnum.TdwgArea);
			}else{
				selectedVocabularies.remove(VocabularyEnum.TdwgArea);
			}
			
			initModel();
		}
	}
	
	private class IncludeWaterbodyOrCountryAction extends Action {
		/**
		 * Creates a new instance of the class.
		 */
		public IncludeWaterbodyOrCountryAction() {
			super("Waterbody Or Country", IAction.AS_CHECK_BOX);
		}

		public void run() {
			if(isChecked()){
				selectedVocabularies.add(VocabularyEnum.WaterbodyOrCountry);
			}else{
				selectedVocabularies.remove(VocabularyEnum.WaterbodyOrCountry);
			}
			
			initModel();
		}
	}
	
	private class IncludeContinentAction extends Action {
		/**
		 * Creates a new instance of the class.
		 */
		public IncludeContinentAction() {
			super("Continent", IAction.AS_CHECK_BOX);
		}

		public void run() {
			if(isChecked()){
				selectedVocabularies.add(VocabularyEnum.Continent);
			}else{
				selectedVocabularies.remove(VocabularyEnum.Continent);
			}
			
			initModel();
		}
	}
	
	private Collection<VocabularyEnum> selectedVocabularies;
	
	private IncludeTdwgAreaAction includeTdwgAreaAction;
	private IncludeWaterbodyOrCountryAction includeWaterbodyOrCountryAction;
	private IncludeContinentAction includeContinentAction;
	
	/**
	 * Creates a filtered selection dialog to select a named area.
	 * 
	 * @param shell
	 * 				The shell for displaying this widget
	 * @param namedArea
	 * 				A namedArea that should be selected when the dialog opens
	 * @return
	 * 				A namedArea object upon selection
	 */
	public static NamedArea selectNamedArea(Shell shell, ConversationHolder conversation, NamedArea namedArea) {
		FilteredNamedAreaSelectionDialog dialog = new FilteredNamedAreaSelectionDialog(shell, conversation,
				"Choose an area", false, namedArea);
		return getSelectionFromDialog(dialog);
	}
	
	protected FilteredNamedAreaSelectionDialog(Shell shell, ConversationHolder conversation, String title, boolean multi, NamedArea namedArea) {
		super(shell, conversation, title, multi, FilteredNamedAreaSelectionDialog.class.getCanonicalName(), namedArea);		
	}
	
	@Override
	protected void fillViewMenu(IMenuManager menuManager) {
				
		super.fillViewMenu(menuManager);
		
		includeTdwgAreaAction = new IncludeTdwgAreaAction();
		menuManager.add(includeTdwgAreaAction);
		includeTdwgAreaAction.setChecked(true);
		
		includeWaterbodyOrCountryAction = new IncludeWaterbodyOrCountryAction();
		menuManager.add(includeWaterbodyOrCountryAction);
		includeWaterbodyOrCountryAction.setChecked(true);
		
		includeContinentAction = new IncludeContinentAction();
		menuManager.add(includeContinentAction);
		includeContinentAction.setChecked(true);
	}
	
	
	@Override
	protected NamedArea getPersistentObject(UUID uuid) {
		for(VocabularyEnum selectedVocabulary : selectedVocabularies){
			TermVocabulary vocabulary = CdmStore.getVocabularyService().getVocabulary(selectedVocabulary);
			for(Object object : vocabulary.getTerms()){
				CdmBase cdmBaseObject = (CdmBase) object;
				if(uuid.equals(cdmBaseObject.getUuid())){
					return (NamedArea) cdmBaseObject;
				}
			}
		}
		return null;
	}
	
	@Override
	protected void init() {
		// testing 
		selectedVocabularies = new HashSet<VocabularyEnum>();
		selectedVocabularies.add(VocabularyEnum.TdwgArea);
		selectedVocabularies.add(VocabularyEnum.WaterbodyOrCountry);
		selectedVocabularies.add(VocabularyEnum.Continent);
	}

	@Override
	protected void initModel() {
		
		Set<NamedArea> terms = new HashSet<NamedArea>();
		for(VocabularyEnum vocabularyEnum : selectedVocabularies){
			TermVocabulary vocabulary = (TermVocabulary) HibernateProxyHelper.deproxy(CdmStore.getVocabularyService().getVocabulary(vocabularyEnum));
			terms.addAll(vocabulary.getTermsOrderedByLabels(CdmStore.getDefaultLanguage()));
		}
		
		if(model == null){
			model = new ArrayList<UuidAndTitleCache<NamedArea>>();
		}
		model.clear();
		for(Object areaObject : terms){
			NamedArea area = (NamedArea) HibernateProxyHelper.deproxy(areaObject);
			UuidAndTitleCache<NamedArea> element = new UuidAndTitleCache<NamedArea>(NamedArea.class, area.getUuid(), getTitle(area)); 
			model.add(element);
		}
	}

	@Override
	protected Control createExtendedContentArea(Composite parent) {
		return null;
	}
	
	@Override
	protected String getTitle(NamedArea namedArea) {
		NamedArea area = (NamedArea) HibernateProxyHelper.deproxy(namedArea);
		
		StringBuilder title = new StringBuilder();
		Representation representation = area.getRepresentation(CdmStore.getDefaultLanguage());
		
		title.append(representation.getDescription());
		title.append(" - ");
		title.append(area.getClass().getSimpleName());
		if(area.getLevel() != null){
			title.append(" - ");
			title.append(area.getLevel().getLabel(CdmStore.getDefaultLanguage()));
		}
		if(! CdmUtils.isEmpty(representation.getAbbreviatedLabel())){
			title.append(" - ");
			title.append(representation.getAbbreviatedLabel());
		}
		
		return title.toString();
	}
	
	@Override
	protected AbstractNewEntityWizard getNewEntityWizard() {
		return null;
	}

	@Override
	protected String getNewWizardLinkText() {
		return null;
	}
}
