/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor;

import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.taxeditor.editor.view.detail.DetailsViewPart;
import eu.etaxonomy.taxeditor.model.AbstractUtility;

/**
 * Utility for the editor package
 * 
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class EditorUtil extends AbstractUtility{

	private static boolean isSaving = false;
	
	public static boolean closeAll() {
		for (IEditorPart editor : getOpenEditors()) {
			if (!getActivePage().closeEditor(editor, true)) {
				return false;
			}
		}
		return true;
	}
	
	/**
	 * Close the given editor.
	 * 
	 * @param editor The <tt>MultipageTaxonEditor</tt> to close.
	 * @return <tt>true</tt> on success
	 */
	public static boolean close(MultiPageTaxonEditor editor) {
		return getActivePage().closeEditor(editor, true);
	}

	/**
	 * Opens a new editor window with the given input
	 * 
	 * @param input
	 * @param editorId
	 * @return 
	 * @throws PartInitException
	 */
	private static void open(final IEditorInput input, final String editorId) throws PartInitException{
		getActivePage().openEditor(input, editorId);
	}
	
	/**
	 * Opens a new editor window with the given TaxonEditorInput 
	 * 
	 * @param input
	 * @return
	 * @throws PartInitException
	 */
	public static void open(TaxonEditorInput input) throws PartInitException{
		open(input, MultiPageTaxonEditor.ID);
	}

	/**
	 * Taxon Editors may be opened by supplying a taxon node uuid.
	 * Session gets initialised here and is passed to the editor
	 * 
	 * @param taxonNodeUuid
	 * @return
	 * @throws PartInitException 
	 */
	public static void openTaxonNode(UUID taxonNodeUuid) throws PartInitException {
		TaxonEditorInput input = TaxonEditorInput.NewInstance(taxonNodeUuid); 
		open(input);
	}
	
	/**
	 * Taxon Editors may be opened by supplying a taxon uuid only.
	 * Session gets initialised here and is passed to the editor
	 * 
	 * @param taxonNodeUuid
	 * @return
	 * @throws PartInitException 
	 */
	public static void openTaxon(UUID taxonUuid) throws PartInitException {
		TaxonEditorInput input = TaxonEditorInput.NewInstanceFromTaxon(taxonUuid); 
		open(input);
	}
	
	public static void openTaxonBase(UUID taxonBaseUuid) throws PartInitException {
		TaxonEditorInput input = TaxonEditorInput.NewInstanceFromTaxonBase(taxonBaseUuid); 
		open(input);
	}
	

	public static IEditorPart findEditorByTaxonNodeUuid(UUID taxonNodeUuid){
		IEditorInput input = TaxonEditorInput.NewInstance(taxonNodeUuid); 
		return getActivePage().findEditor(input);
	}
	
	/**
	 * An uninitialized taxon is one that hasn't been saved yet. As such, it should appear in neither
	 * the list of recent names nor in the taxonomic tree when opened.
	 * 
	 * @param parentTaxon 	The UUID of the parental taxon or <code>null</code> if this is
	 * 						supposed to be a root taxon.
	 * @return 				The EditorPart.
	 * @throws PartInitException 
	 */
	public static void openEmpty(UUID parentNodeUuid) throws PartInitException{
		TaxonEditorInput input = TaxonEditorInput.NewEmptyInstance(parentNodeUuid);
		open(input, MultiPageTaxonEditor.ID);
		
		getActiveMultiPageTaxonEditor().changed(null);
		
		
	}

	public static void setSaving(boolean isSaving) {
		EditorUtil.isSaving = isSaving;
	}

	public static boolean isSaving() {
		return isSaving;
	}
	
	/**
	 * Returns a set of all currently open 
	 * <code>MultiPageTaxonEditor</code>s.
	 * 
	 * @return
	 */
	public static Set<IEditorPart> getOpenEditors() {
		Set<IEditorPart> taxonEditors = new HashSet<IEditorPart>();
		
		if(getActivePage() != null){
			for (IEditorReference reference : getActivePage().getEditorReferences()) {
				IEditorPart editor = reference.getEditor(false);
				if (editor instanceof MultiPageTaxonEditor) {
					taxonEditors.add(editor);
				}
			}
		}
		
		return taxonEditors;
	}
	
	public static IEditorPart getActiveEditor(){
		return getActivePage() != null ? getActivePage().getActiveEditor() : null;
	}
	
	/**
	 * Returns the currently active taxon editor
	 * 
	 * @return the taxon editor that has focus
	 */
	public static MultiPageTaxonEditor getActiveMultiPageTaxonEditor(){
		IEditorPart editorPart = getActiveEditor();
		if(editorPart != null && editorPart instanceof MultiPageTaxonEditor){
			MultiPageTaxonEditor editor = (MultiPageTaxonEditor) editorPart;
			editor.getConversationHolder().bind();
			return editor;
		}
		return null;
	}
	
	public static IEditorPart getActiveEditorPage(Page page){
		MultiPageTaxonEditor editor = getActiveMultiPageTaxonEditor();
		
		return editor.getPage(page);
	}
	
	/**
	 * Returns the selection of the currently active taxon editor
	 * 
	 * @return
	 */
	public static ISelection getCurrentSelection(){
		if(getActiveMultiPageTaxonEditor() == null){
			return null;
		}else{
			return getActiveMultiPageTaxonEditor().getSite().getSelectionProvider().getSelection();
		}
	}

	public static IUndoContext getUndoContext(MultiPageTaxonEditor editor){
		return editor.getUndoContext();
	}
	
	public static IUndoContext getUndoContext() {
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}


	/**
	 * @param editor
	 * @param shell 
	 * @return 
	 */
	public static boolean forceUserSave(IEditorPart editor, Shell shell) {
		if (editor.isDirty()) {
			
			boolean doSave = MessageDialog.openConfirm(shell, "Confirm save", "The current editor must be saved before this action can be executed.");
			
			if (!doSave) {
				return false;
			}
			
			editor.doSave(null);
		}
		return true;
	}
	
	public static DetailsViewPart getDetailsView(){
		return  (DetailsViewPart) EditorUtil.showView(DetailsViewPart.ID);
	}
	
	public static void refreshDetailsViewer(){
		getDetailsView().getDetailsViewer().refresh();
	}
	
	public static IStructuredSelection getSelection(ExecutionEvent event){
		IEditorPart activeEditor = HandlerUtil.getActiveEditor(event);
				
		return (IStructuredSelection) activeEditor.getSite().getSelectionProvider().getSelection();
	}
}
