/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.name;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetListener;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.CdmDataTransfer;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.IDropTargetable;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 25.06.2008
 * @version 1.0
 */
public abstract class AbstractGroup<T extends TaxonBase> implements IDropTargetable{
	private static final Logger logger = Logger.getLogger(AbstractGroup.class);
	
	private Composite control;
	
	private List<AbstractGroupedContainer<T>> groupedContainers = new ArrayList<AbstractGroupedContainer<T>>(); 
	
	protected TaxonNameEditor editor;
	
	private DropTarget target;
	private DropTargetListener dropListener;

	public AbstractGroup(TaxonNameEditor editor){
		this.editor = editor;
		
		createControl();
		
		// Drop functionality
		this.setDroppable(true);

	}
	
	protected void createControl() {
		control = editor.getToolkit().createComposite(editor.getControl());
		
		control.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		TableWrapLayout layout = new TableWrapLayout();
		
		layout.topMargin = 0;
		layout.rightMargin = 0;
		layout.bottomMargin = 1;
		layout.leftMargin = 0;
		
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		control.setLayout(layout);
	
		control.setBackground(Display.getDefault().getSystemColor(SWT.COLOR_GRAY));
	}
	
	public void add(AbstractGroupedContainer groupedContainer){
		groupedContainers.add(groupedContainer);
		groupedContainer.setGroup(this);
	}
	
	public void remove(AbstractGroupedContainer groupedContainer){
		deleteContainer(groupedContainer);
		groupedContainers.remove(groupedContainer);
		
		editor.getManagedForm().reflow(true);
	}
	
	public void removeAllContainers() {
		for(AbstractGroupedContainer container : groupedContainers){
			deleteContainer(container);
		}
		
		groupedContainers.clear();
	}
	
	public void clearContainers(){
		groupedContainers.clear();
	}
	
	public void deleteContainer(AbstractGroupedContainer groupedContainer){
		groupedContainer.setMenu(null);
		groupedContainer.getControl().dispose();
	}
	
	public void clear(){
		control.setMenu(null);
		control.dispose();
	}
	
	public List<AbstractGroupedContainer<T>> getGroupedContainers(){
		return groupedContainers;
	}
	
	
	public void setDroppable(boolean droppable) {
		if (droppable) {
			Transfer[] types = new Transfer[] {CdmDataTransfer.getInstance()};	
			int operations = DND.DROP_MOVE;
			target = new DropTarget(control, operations);
			target.setTransfer(types);
			dropListener = new NameEditorDropTargetListener(this);
			target.addDropListener(dropListener);
			
			target.setDropTargetEffect(new NameEditorDropTargetEffect(control));
			
		} else {
			if (dropListener != null) {
				target.removeDropListener(dropListener);
			}
		}
	}
	
	public Composite getControl(){
		return control;
	}

	public TaxonNameEditor getNameEditor() {
		return editor;
	}

	public void dragEntered() {
		Color color = EditorUtil.getColor(Resources.COLOR_DRAG_ENTER);
		
		for(AbstractGroupedContainer container : groupedContainers){
			container.setBackground(color);
		}		
	}

	public void dragLeft() {
		for(AbstractGroupedContainer container : groupedContainers){
			container.restoreColor();
		}
	}
	
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		editor.getMultiPageTaxonEditor().changed(null);
		redraw();
		return true;
	}

	public abstract void redraw();
}
