/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.name;

import java.lang.reflect.Method;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextViewer;
import org.eclipse.jface.window.DefaultToolTip;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ControlListener;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.forms.widgets.TableWrapData;
import org.eclipse.ui.forms.widgets.TableWrapLayout;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.CdmDataTransfer;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.labels.ILabelImageStrategy;
import eu.etaxonomy.taxeditor.labels.LabelImageProvider;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.model.TextHelper;
import eu.etaxonomy.taxeditor.operations.CreateSynonymInNewGroupOperation;
import eu.etaxonomy.taxeditor.parser.ParseHandler;
import eu.etaxonomy.taxeditor.preference.Resources;

/** 
 * Formats <code>GroupedComposite</code> with cosmetic and layout properties specific to the
 * Editor. This should be used to maintain a consistent look and feel for all Editor 
 * freetext area components, such as DescriptionElementComposite.
 * <p>
 * Requires an <code>IManagedForm</code>, whose <code>input</code> is set to the contents
 * of {@link #getData()} when the <code>GroupedComposite</code> gets focus, i.e. to
 * populate the property sheet with the data.
 * </p>
 * <p>
 * The <code>IManagedForm</code> is also required to have a <code>Taxon</code> in its 
 * own <code>getData()</code>.
 * </p> 
 * <p>
 * The <code>IManagedForm</code> can also used for drawing borders by calling the method 
 * <code>createBorderSupport()</code>.
 * </p>
 * @author p.ciardelli
 * @author n.hoffmann
 * @created 02.06.2008
 * @version 1.0
 */
abstract public class AbstractGroupedContainer<T extends TaxonBase> {	
	private static final Logger logger = Logger.getLogger(AbstractGroupedContainer.class);
	
	/**
	 * 
	 */
	public static final int SELECTED_FOCUS = 1;
	public static final int SELECTED_NO_FOCUS = 2;
	public static final int NOT_SELECTED = 3;
	
	/**
	 * ************ COMPOSITE TYPES ************
	 */
	public String compositeType;
	public static final String ACCEPTED_TAXON = "accepted_name_composite";
	public static final String HOMOTYPIC_SYNONYM = "homotypic_name_composite";
	public static final String HETEROTYPIC_SYNONYM = "heterotypic_name_composite";
	public static final String MISAPPLIED_NAME = "misappliedname_name_composite";
	public static final String CONCEPTRELATION = "concept_name_composite";

	/**
	 * ************ INDENTATIONS ************
	 */
	public static final int ACCEPTED_INDENT = 20;
	public static final int SYNONYM_INDENT = 40;
	public static final int MISAPPLIEDNAME_INDENT = 40;
	public static final int CONCEPT_INDENT = 40;

	
	/**
	 * ************ ICONS ************
	 */
	public static final Image ACCEPTED_ICON = ImageResources.getImage(ImageResources.BLACK_SQUARE_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ICON = ImageResources.getImage(ImageResources.HOMOTYPIC_SYN_ICON);
	public static final Image HOMOTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = ImageResources.getImage(
			ImageResources.HOMOTYPIC_SYN_ORIGINAL_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ICON = ImageResources.getImage(ImageResources.HETEROTYPIC_SYN_ICON);
	public static final Image HETEROTYPIC_SYNONYM_ORIGINAL_COMBINATION_ICON = ImageResources.getImage(
			ImageResources.HETEROTYPIC_SYN_ORIGINAL_ICON);
	public static final Image MISAPPLIEDNAME_ICON = ImageResources.getImage(ImageResources.MISAPPLIED_NAME_ICON);
	public static final Image CONCEPT_ICON = ImageResources.getImage(ImageResources.CONCEPT_ICON);
	public static final Image AUTONYM_ICON = ImageResources
			.getImage(ImageResources.AUTONYM_ICON);
	public static final Image MOVE = ImageResources.getImage(ImageResources.MOVE_ICON);

	/**
	 * ************ TRANSFORMATIONS ************
	 */
	public static final String ADD_GROUP_BASIONYM = "add_group_basionym";
	public static final String REMOVE_GROUP_BASIONYM = "remove_group_basionym";

	private static final String EMPTY_NAME_PROMPT = "Click to add name";
	
	protected ParseHandler parseHandler;
	
	private FocusListener nameCompositeFocusListener;
	private ModifyListener nameCompositeModifyListener;
	
	final Color not_editable_color = new Color(Display.getDefault(), 105, 114, 124);

	/**
	 *  1st column: Return value expected from the last method mentioned in the object path (3rd column et seq.) to enable the freetext.
	 *  2nd column: The class the last method mentioned in the object path should be a member of. This should be null if there is no need to narrow this.
	 *  3rd column et seq.: The sequence of method names reflects the object path to the desired method starting from a member of TaxonBase.
	 */
	private Object[][] enableFreetextFor = {
			{"", null, "getName", "getAppendedPhrase"},
			{false, null, "getName", "isProtectedAuthorshipCache"},
			{false, null, "getName", "isProtectedNameCache"},
//			{false, "eu.etaxonomy.cdm.model.agent.Team", "getName", "getCombinationAuthorTeam", "isProtectedNomenclaturalTitleCache"}
	};

	
	
	protected TaxonNameEditor editor;
	
	protected NameViewer textViewer;
	
	private AbstractGroup<T> group;
	
	private Label nonEditableInfoLabel;
	private DefaultToolTip nonEditableInfoHover;
	
	private static AbstractGroupedContainer selection;
	
	private FocusListener focusListener;
	private LineBreakListener lineBreakListener;

	private int cursorPosition;

	protected Composite control;

	private Color backgroundColor;
	
	public AbstractGroupedContainer(TaxonNameEditor editor, AbstractGroup<T> group, T taxonBase){
		this.editor = editor;
		
		this.group = group;	
		group.add(this);
		
		setData( (T) HibernateProxyHelper.deproxy(taxonBase));
		
		createControl();
		
		createTextViewer();
		createLineWrapSupport();
		createLineBreakListener();

		setMenu(editor.getMenu());
		
		setDraggableControl(textViewer.getRulerControl());
		
		createEmptyViewerPrompt(EMPTY_NAME_PROMPT);
		
		parseHandler = ParseHandler.NewInstance(textViewer.getTextWidget(), taxonBase.getName());
		
		initializeComposite();
		
		createListener();

		isFreeTextEnabled();
		
		logger.trace(this.getClass().getSimpleName() + " created");
	}

	
	
	protected void createListener(){
		nameCompositeModifyListener = new ModifyListener(){
			
			public void modifyText(ModifyEvent e) {
				// mark the composite dirty
				setDirty(true);
				// parse the text
				NonViralName name = parseHandler.parse();
				getTaxonBase().setName(name);
				getTaxonBase().setTitleCache((getTaxonBase().generateTitle()));
				
				// show errors resulting from parsing
				calculateErrors();
				// store the position of the cursor				
				storeCursor();
				// notify selection listener
				setDelayedSelection();
			}
		};
		nameCompositeFocusListener = new FocusAdapter(){

			/* (non-Javadoc)
			 * @see org.eclipse.swt.events.FocusAdapter#focusLost(org.eclipse.swt.events.FocusEvent)
			 */
			@Override
			public void focusLost(FocusEvent e) {
				super.focusLost(e);

				persistName();
			}		
		};
		
		addListener();
	}
	
	private void addListener(){
		textViewer.getTextWidget().addModifyListener(nameCompositeModifyListener);
		textViewer.getTextWidget().addFocusListener(nameCompositeFocusListener);
	}
	
	private void removeListener(){
		textViewer.getTextWidget().removeModifyListener(nameCompositeModifyListener);
		textViewer.getTextWidget().removeFocusListener(nameCompositeFocusListener);
	}
		
	/**
	 * Initialize the composite specific code
	 * 
	 */
	protected abstract void initializeComposite();
	
	protected String getEmptyTextPrompt() {
		return EMPTY_NAME_PROMPT;
	}
	
	/**
	 * 
	 */
	private void showNameRelations() {
		TaxonNameBase<?, ?> name = getName();
		if (name == null) {
			return;
		}
		
		ILabelImageStrategy strategy = LabelImageProvider.getLabelStrategy(name);
		LabelImageProvider labelProvider = new LabelImageProvider(strategy);
		
		Set<NameRelationship> nameRelations = name.getNameRelations();
		if (nameRelations.size() == 0) {
			return;
		}
		for (NameRelationship nameRelation : nameRelations) {
			String typeLabel = null;
			TaxonNameBase<?, ?> relatedName = null; 

			if (name.equals(nameRelation.getFromName())) {
				typeLabel = labelProvider.getNameRelationTypeLabel( 
								nameRelation.getType());
				relatedName = nameRelation.getToName();
			} else {
				typeLabel = labelProvider.getNameRelationTypeInverseLabel( 
								nameRelation.getType());
				relatedName = nameRelation.getFromName();
			}
			
//			setNonEditableInfo(typeLabel + " " + NameHelper.getDisplayName(relatedName));
		}
	}
		
	protected void initTextViewer() {
		
		showNameRelations();
		
		String text = NameHelper.getDisplayNameWithRef(getData());
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			textViewer.setText(text);
			placeCursor();
		}
		calculateErrors();
	}
	
	synchronized protected void calculateErrors() {
		textViewer.clearErrors();
		textViewer.setShowParsingErrors(getName());
	}

	protected void handleSplitText(String text) {
		// Create a synonym in a new homotypic group using text as name
		TaxonNameBase synonymName = ParseHandler.quickParse(text);
		
		EditorUtil.executeOperation(new CreateSynonymInNewGroupOperation
				("New Heterotypic Synonym", editor.getUndoContext(), editor.getTaxon(), synonymName, editor));
		logger.debug("new heterotypic synonym created through entering of line break");
	}

	/**
	 * Refreshes the display with latest data from the model.
	 * 
	 * Note: Will not parse the text and not calculate errors! 
	 * 
	 */
	public void refresh() {
		showNameRelations();
		
		String text = NameHelper.getDisplayNameWithRef(getTaxonBase());
		
		if (text.length() == 0) {
			initEmptyText();
		} else {
			removeListener();
			textViewer.getTextWidget().setText(text);
			addListener();
		}
		
		updateNonEditableInfo();
		
//		placeCursor();
		
		isFreeTextEnabled();
	}

	protected abstract void updateNonEditableInfo();
	
	/**
	 * 
	 */
	private void isFreeTextEnabled() {
		// Decide whether editing of freetext is allowed or not.
		if (freetextEditingAllowed()) {
			
			// set editable
			textViewer.getTextWidget().setEditable(true);
			
			// restore foreground font color again
			textViewer.getTextWidget().setForeground(control.getForeground());
		} else {
			// set non-editable
			textViewer.getTextWidget().setEditable(false);
			
			// grey out text as a non-editable indicator
			textViewer.getTextWidget().setForeground(not_editable_color);
		}
	}

	/**
	 * Checks whether the freetext should be editable based on specific empty fields.
	 * @return
	 */
	private boolean freetextEditingAllowed() {
		boolean enableFreetext = true;

		enableFreetext &= taxonFieldsEmpty();

		return enableFreetext;
	}

	/**
	 * Checks whether specific fields of specific type have specific values.
	 * This generic implementation can easily go completely nuts if enableFreetextFor was filled wrong.
	 * @return
	 */
	private boolean taxonFieldsEmpty() {
		boolean enableFreetext = true;

		if (getTaxonBase() != null) {
			for (Object[] methodSequence : enableFreetextFor) {
				int sequenceCount = 0;
				Object taxonObject = HibernateProxyHelper.deproxy(getTaxonBase());
				Object referenceType = null;
				boolean useReference = false;
				Class<? extends CdmBase> referenceInstance = null;
				for (Object method : methodSequence) {
					sequenceCount++;

					if (sequenceCount == 1) {
						referenceType = method;
					} else if (sequenceCount == 2) {
						try {
							if (method != null) {
								referenceInstance = (Class<? extends CdmBase>) Class.forName((String) method);
								useReference = true;
							} else {
								useReference = false;
							}
						} catch (ClassNotFoundException e) {
							logger.error("Class " + method + " could not be found: ", e);
							return false;
						}
					} else {
						Method getter = null;
						try {
							if (taxonObject != null) { // authorTeams can be null
								Class<? extends Object> clazz = taxonObject.getClass();
								boolean goOn = false;
								if (methodSequence.length == sequenceCount) {
									if (useReference) {
										if (referenceInstance.newInstance().isInstanceOf((Class<? extends CdmBase>) clazz)) {
											goOn = true;
										} else {
											goOn = false;
										}
									} else {
										goOn = true;
									}
								} else {
									goOn = true;
								}
								
								if (goOn) {
									getter = clazz.getMethod((String)method, (Class<?>[])null);
									taxonObject = HibernateProxyHelper.deproxy(getter.invoke(taxonObject, (Object[])null));
									if (methodSequence.length == sequenceCount) {
										enableFreetext &= isEqual(referenceType, taxonObject);
//										if (!enableFreetext) {
//											logger.error("Freetext disabled for: " + taxonBase);
//										}
									}
								}
							}
						} catch (Exception e) {
							e.printStackTrace();
							logger.error("Method " + method + " could not be called: " + e.getMessage());
							return false;
						}
					}
				}
			}
		} else {
			logger.warn("TaxonBase is null. This should not happen.");
		}

		return enableFreetext;
	}

	/**
	 * Checks whether two objects are equal. Only works for cases of interest.
	 * @param referenceType
	 * @param object
	 * @return
	 */
	private boolean isEqual(Object referenceType, Object object) {
		boolean equal = false;
		if (referenceType instanceof Boolean) {
			if ((Boolean)referenceType.equals((Boolean)object)) {
				equal = true;
			} else {
				equal = false;
			}
		} else {
			// This can only be a String in our cases so far. Needs reimplementation in case this is not always the case.
			if (referenceType.equals(CdmUtils.Nz((String)object))) {
				equal = true;
			} else {
				equal = false;
			}
		}
		return equal;
	}

	/**
	 * Parse the text and calculate errors
	 * 
	 */
	public void parseAndCalculateErrors(){
		removeListener();
		parseHandler.parse();
		addListener();
		calculateErrors();
	}

	/**
	 * @return the taxonBase
	 */
	public T getTaxonBase() {
		return getData();
	}
	
	public TaxonNameBase getName(){
		return getTaxonBase().getName();
	}

	public void persistName(){
		if(isDirty()){
			logger.warn("Found dirty composite: " + this + ". Name: " + textViewer.getTextWidget().getText());
			getTaxonBase().setName(parseHandler.parseAndResolveDuplicates());
			
			getTaxonBase().setTitleCache((getTaxonBase().generateTitle()));

		}
	}
	
	public AbstractGroup getGroup(){
		return group;
	}
	
	public void remove(){
		getGroup().remove(this);
	}
		
	protected void createControl() {
		control = editor.getToolkit().createComposite(group.getControl());
		
		control.setLayoutData(new TableWrapData(TableWrapData.FILL_GRAB));
		TableWrapLayout layout = new TableWrapLayout();
		layout.leftMargin = 0;
		layout.rightMargin = 0;
		layout.topMargin = 5;
		layout.bottomMargin = 5;
		
		layout.verticalSpacing = 0;
		layout.horizontalSpacing = 0;
		
		control.setLayout(layout);
		
		setBackground(EditorUtil.getColor(Resources.COLOR_COMPOSITE_BACKGROUND));
	}
	
	public Composite getControl(){
		return control;
	}
	
	protected void createLineWrapSupport() {
		if (textViewer instanceof NameViewer) {
			new LineWrapSupport(textViewer, editor.getManagedForm());
		} else {
			logger.warn("Can't create line wrap support because textViewer has not been initialized.");
		}
	}
	
	
	protected void createTextViewer() {
		textViewer = new NameViewer(control);
		
		focusListener = new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				for(AbstractGroupedContainer container : editor.getGroupedContainers()){
					container.colorSelected(NOT_SELECTED);
				}
				editor.getManagedForm().setInput(AbstractGroupedContainer.this);
//				setSelected();
				placeCursor();
				colorSelected(SELECTED_FOCUS);
				logger.warn("TextViewer gained focus: " + AbstractGroupedContainer.this);
			}
			public void focusLost(FocusEvent e) {
//				colorSelected(NOT_SELECTED);
				logger.warn("TextViewer lost focus: " + AbstractGroupedContainer.this);
				logger.warn("------------");
			};
		};
		textViewer.getTextWidget().addFocusListener(focusListener);
		
//		
		MouseAdapter mouseListener = new MouseAdapter() {
			public void mouseDown(MouseEvent e) {
				storeCursor();
//				setFocus();		
				
//				logger.warn("mouseDown");
//				logger.warn(textViewer.getCursorPosition());
			}
		};
		control.addMouseListener(mouseListener);
		textViewer.getRulerControl().addMouseListener(mouseListener);
		textViewer.getTextWidget().addMouseListener(mouseListener);
	}
	
	/**
	 * @param icon
	 */
	public void setIcon(Image icon) {
		if (textViewer instanceof NameViewer) {
			textViewer.setIcon(icon);
		} else {
			logger.warn("Can't set icon because textViewer has not been initialized.");
		}
	}
	
	/**
	 * @param indent
	 */
	public void setIndent(int indent) {
		if (control.getLayout() instanceof TableWrapLayout) {
			TableWrapLayout layout = ((TableWrapLayout) control.getLayout());
			layout.leftMargin = indent;
			layout.rightMargin = ACCEPTED_INDENT;
			control.setLayout(layout);
		} else {
			logger.warn("Couldn't indent - composite's layout must be TableWrapLayout.");
		}
	}	
	
	/**
	 * 
	 * @return
	 */
	public void setSelected() {
		textViewer.getTextWidget().setFocus();
	}
	
	public boolean isSelected(){
		return editor.getSelectedObject() == this;
	}
	
	public void colorSelected(int mode){
		if(!control.isDisposed()){
			String colorString = null;
			
			switch(mode){
			case SELECTED_FOCUS:
				colorString = Resources.COLOR_CONTROL_SELECTED_FOCUS;
				break;
			case SELECTED_NO_FOCUS:
				colorString = Resources.COLOR_CONTROL_SELECTED;
				break;
			default:
				colorString = Resources.COLOR_COMPOSITE_BACKGROUND;
			}
			
			backgroundColor = EditorUtil.getColor(colorString);
			
			setBackground(backgroundColor);
		}
	}

	
	/**
	 * 
	 */
	protected void setDelayedSelection(){
		//TODO this might be done better
		// this is the quickest solution i could come up with and it improves performance
		// please reimplement if you know better.
		selection = this;
		
		// start timer
		Display display = Display.getCurrent();
		Runnable runnable = new Runnable() {
			
			public void run() {
				editor.getManagedForm().setInput(selection);
			}
		};
		display.timerExec(1000, runnable);
		
	}
	
	/**
	 * 
	 */
	public void setBackground(Color color) {
		control.setBackground(color);
		
		for(Control child : control.getChildren()){
			child.setBackground(color);
		}
		
		if(textViewer != null){
			textViewer.setBackground(color);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.widgets.Control#setFont(org.eclipse.swt.graphics.Font)
	 */
	public void setFont(Font font) {
		if (textViewer != null) {
			textViewer.getTextWidget().setFont(font);
		} else {
			logger.warn("Can't set font because textViewer has not been initalized.");
		}
	}
	
	public TextViewer getTextViewer() {
		return textViewer;
	}
	
	/**
	 * If <code>textViewer</code> has already been set, it will show a 
	 * <code>prompt</code> along the lines of "Click here to start entering data" 
	 * when empty.
	 * 
	 * @param prompt
	 */
	public void createEmptyViewerPrompt(final String prompt) {
		
		Assert.isNotNull(textViewer);
		 
		final StyledText textControl = textViewer.getTextWidget();
		final IDocument document = textViewer.getDocument();
		
		setFocusListener(new FocusListener() {

			
			public void focusGained(FocusEvent e) {
				if (document.get().equals(prompt)) {
					textControl.setFont(getViewerFont());
					document.set("");
				}
			}

			
			public void focusLost(FocusEvent e) {
				if (document.getLength() == 0) {
					initEmptyText();
				}
			}
			
		});
		textControl.addFocusListener(getFocusListener());
		
		if (document.getLength() == 0) {
			textControl.setFont(EditorUtil.getFont(Resources.FONT_DEFAULT_PROMPT));
			document.set(prompt);
		}
	}
	
	abstract protected Font getViewerFont();

	protected void initEmptyText() {
		Font defaultFont = EditorUtil.getFont(Resources.FONT_DEFAULT_PROMPT);
		textViewer.getTextWidget().setFont(defaultFont);
		
		textViewer.getDocument().set(getEmptyTextPrompt());
		placeCursor();
	}

	protected void setFocusListener(FocusListener focusListener) {
		this.focusListener = focusListener;
	}

	private FocusListener getFocusListener() {
		return focusListener;
	}

	/**
	 * @param isDirty
	 */
	public void setDirty(boolean isDirty) {
		editor.getManagedForm().dirtyStateChanged();
	}
	
	public boolean isDirty(){
		return editor.getManagedForm().isDirty();
	}
	
	public void setMenu (Menu menu) {
		control.setMenu(menu);
		
		if (textViewer != null) {
			textViewer.setMenu(menu);
		}
	}
	
	private Control draggableControl;
	private DragSource dragSource;
	
	protected void setDraggableControl(Control control) {
		draggableControl = control;
	}

	public DragSource getDragSource() {
		return dragSource;
	}
	
	public void setIsDraggable(boolean draggable) {

		if (draggable) {

			if (dragSource != null) {
				// Already initialized
				return;
			}

			if (draggableControl == null) {
				throw new NullPointerException(
						"Draggable control must be set to add draggability");
			}
			
			Transfer[] types = new Transfer[] { CdmDataTransfer.getInstance() };			
			int operations = DND.DROP_MOVE;

			dragSource = new DragSource(draggableControl, operations);
			dragSource.setTransfer(types);
			
			dragSource.addDragListener(new NameEditorDragListener(this));
			dragSource.setDragSourceEffect(new NameEditorDragSourceEffect(control));
		} else {
			dragSource = null;
		}
	}

	private String nonEditableText;

	ControlListener nonEditableResizeListener = new ControlAdapter() {
		
		int width = 0;
		
		public void controlResized(ControlEvent e) {
			if (nonEditableInfoLabel.getBounds().width == width) {
				return;
			}
			width = nonEditableInfoLabel.getBounds().width;
			if (nonEditableInfoLabel.getBounds().width > 0) {
				nonEditableInfoLabel.setText(
						Dialog.shortenText(nonEditableText.toUpperCase(), 
						nonEditableInfoLabel));
			}
		}
	};

	private String nonEditableHoverText;

	private LabelEllipsisListener nonEditableLabelEllipsisListener;

	private T data;
			
	/**
	 * nonEditableInfo is a label displayed underneath a GroupedComposite's 
	 * input field. For instance, NameComposites display things like name relations,
	 * sec. references, etc. here.
	 * 
	 * @param info the text to display in the label
	 * @param append whether the string should be appended to text that is already shown in the label
	 */
	public void setNonEditableInfo(String info, boolean append) {
		// TODO non editable info should only be drawn once, when everything else is drawn
		info = info.toUpperCase();
		
		if(append){
			nonEditableText += ", " + info;
			nonEditableHoverText += "\n" + info;
		}else{
			nonEditableText = info;
			nonEditableHoverText = info;
		}
		
		if (nonEditableInfoLabel == null) {
			nonEditableInfoLabel = editor.getToolkit().createLabel(control, "");
			TableWrapData layoutData = new TableWrapData(TableWrapData.FILL_GRAB, TableWrapData.TOP);
			// Set indent to viewer ruler's width 
			if (textViewer != null && textViewer.getRulerControl() != null) {
				// TODO right justify
				layoutData.indent = NameViewer.RULER_WIDTH;
			}
			nonEditableInfoLabel.setLayoutData(layoutData);
			

			
			nonEditableLabelEllipsisListener = new LabelEllipsisListener(nonEditableInfoLabel) {
				@Override
				public String getLabelText() {
					return nonEditableText.toUpperCase();
				}
			}; 
			nonEditableInfoLabel.addControlListener(nonEditableLabelEllipsisListener);
			
			nonEditableInfoHover = new DefaultToolTip(nonEditableInfoLabel);
			nonEditableInfoHover.setRespectDisplayBounds(true);
			
		} 
		nonEditableInfoHover.setText(nonEditableHoverText);
		nonEditableInfoLabel.setText(nonEditableText);
	}
	
	public T getData(){
		return data;
	}
	
	public void setData(T data){
		this.data = data;
	}

	/**
	 * If the user hitting carriage return should cause something to happen -
	 * i.e. the creation of a new composite - call this method and override
	 * the method handleSplitText().
	 */
	protected void createLineBreakListener() {
		if (textViewer != null) {
			lineBreakListener = new LineBreakListener() {
				@Override
				public void handleSplitText(String text) {
					AbstractGroupedContainer.this.handleSplitText(text);	
				}
			};
			
			textViewer.getTextWidget().addVerifyListener(lineBreakListener);
			textViewer.getTextWidget().addKeyListener(lineBreakListener);
		} else {
			logger.warn("Can't create line break listener because textViewer has not been initalized.");
		}
	}
		
	abstract class LabelEllipsisListener extends ControlAdapter {
		
		private Label label;
		int width = 0;

		LabelEllipsisListener(Label label) {
			this.label = label;
		}
		
		abstract public String getLabelText();
		
		public void controlResized(ControlEvent e) {
			if (label.getBounds().width == width) {
				return;
			}
			width = label.getBounds().width;
			if (label.getBounds().width > 0) {
				label.setText(TextHelper.shortenText(getLabelText(), label));
			}
		}
	}

	public void storeCursor() {
		this.cursorPosition = textViewer.getCursorPosition();
	}
	
	/**
	 * Puts the cursor to the position it was last seen on or to the end of line
	 * if no former position is known.
	 */
	public void placeCursor(){
		if(cursorPosition == 0){
			textViewer.setCursorToEOL();
		}else{
			textViewer.setCursorPosition(cursorPosition);
		}
	}



	public void setGroup(AbstractGroup<T> group) {
		this.group = group;
	}



	public void restoreColor() {
		setBackground(backgroundColor);
	}
} 