/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;

/**
 * @author p.ciardelli
 *
 */
public class DescriptiveContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(DescriptiveContentProvider.class);

	private static final Object[] NO_CHILDREN = new Object[0];
		
	private Map<TaxonDescription, Map<Feature, TaxonDescriptionFeature>> descriptionFeatureCache;
		
	public DescriptiveContentProvider(Map<TaxonDescription, Map<Feature, TaxonDescriptionFeature>> descriptionFeatureCache){
		this.descriptionFeatureCache = descriptionFeatureCache;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		if (parentElement instanceof TaxonEditorInput) {
			Taxon taxon = ((TaxonEditorInput) parentElement).getTaxon();
			HashSet<DescriptionBase> descriptions = new HashSet<DescriptionBase>();
			for(DescriptionBase description : taxon.getDescriptions()){
				if(! description.isImageGallery()){
					descriptions.add(description);
				}
			}			
			return descriptions.toArray(); 
		}
		if (parentElement instanceof TaxonDescription) {
			if ( ! ((TaxonDescription) parentElement).isImageGallery()) {
				return getDescriptionFeatures((TaxonDescription) parentElement).toArray();
			}
		}
		if (parentElement instanceof TaxonDescriptionFeature){
			TaxonDescriptionFeature taxonDescriptionFeature = (TaxonDescriptionFeature) parentElement;
			Feature feature = taxonDescriptionFeature.getFeature();
			TaxonDescription description = taxonDescriptionFeature.getDescription();
			return getDescriptionElements(description, feature).toArray();
		}
		
		return NO_CHILDREN;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		return (getChildren(element).length > 0);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {
		return getChildren(inputElement);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
	
	
	private List<TaxonDescriptionFeature> getDescriptionFeatures(TaxonDescription parentElement) {
		
		Map<Feature, TaxonDescriptionFeature> featureCache = getFeatureCache(parentElement);
		
		Set<Feature> featuresTemp = new HashSet<Feature>();
		
		Set<DescriptionElementBase> elements = parentElement.getElements();
		if (elements != null) {
			for (DescriptionElementBase element : elements) {
				Feature feature = HibernateProxyHelper.deproxy(element.getFeature(), Feature.class);
				featuresTemp.add(feature);
				
				// Is there already an entry in map for this Feature?
				if (!(featureCache.containsKey(feature))) {
					featureCache.put(feature, new TaxonDescriptionFeature(parentElement, feature));
				}				
			}
		}
		
		// Check whether any features need to be removed, i.e. their elements were deleted
		Set<Feature> keys = featureCache.keySet();
		HashSet<Feature> removeFeatures = new HashSet<Feature>();
		for (Feature feature : keys) {
			if (!featuresTemp.contains(feature)) {
				removeFeatures.add(feature);
			}
		}
		for (Feature feature : removeFeatures) {
			featureCache.remove(feature);
		}
		
		return new ArrayList<TaxonDescriptionFeature>(featureCache.values());
	}
	
	private Map<Feature, TaxonDescriptionFeature> getFeatureCache(TaxonDescription description) {
		Map<Feature, TaxonDescriptionFeature> featureCache = descriptionFeatureCache.get(description);
		if (featureCache == null) {
			featureCache = new HashMap<Feature, TaxonDescriptionFeature>();
			descriptionFeatureCache.put(description, featureCache);
		}
		return featureCache;
	}
	
	private List<DescriptionElementBase> getDescriptionElements(
			DescriptionBase description, Feature feature) {
		List<DescriptionElementBase> featureElements = new ArrayList<DescriptionElementBase>();
		
		Set<DescriptionElementBase> elements = description.getElements();
		if (elements != null) {
			for (DescriptionElementBase element : elements) {
				if (feature.equals(element.getFeature())) {
					featureElements.add(element);
				}
			}
		}
		
		Collections.sort(featureElements, new Comparator(){

			public int compare(Object o1, Object o2) {
				return DescriptiveLabelProvider.getLabel(o1).compareTo(DescriptiveLabelProvider.getLabel(o2));
			}
			
		});
		
		return featureElements;
	}
}