// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.descriptive;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.taxeditor.bulkeditor.BulkEditor;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.model.IPartContentHasDetails;
import eu.etaxonomy.taxeditor.model.IPartContentHasSupplementalData;
import eu.etaxonomy.taxeditor.model.ImageResources;

/**
 * @author n.hoffmann
 * @created Jun 9, 2010
 * @version 1.0
 */
public class DescriptiveViewPart extends AbstractCdmViewPart implements IPartContentHasDetails, IPartContentHasSupplementalData {
	private static final Logger logger = Logger
			.getLogger(DescriptiveViewPart.class);

	public static final String ID = "eu.etaxonomy.taxeditor.editor.view.description";

	private Map<TaxonDescription, Map<Feature, TaxonDescriptionFeature>> descriptionFeatureCache;
	
	private TreeViewer descriptionViewer;

	private ToggleDescriptionAction showAllElementsAction;

	private ToggleDescriptionAction hideAllElementsAction;
		
	@Override
	public void createViewer(Composite parent) {
		
		descriptionFeatureCache = new HashMap<TaxonDescription, Map<Feature, TaxonDescriptionFeature>>();
		
		descriptionViewer = new TreeViewer(new Tree(parent, SWT.NULL));
				
		descriptionViewer.setContentProvider(new DescriptiveContentProvider(descriptionFeatureCache));		
		descriptionViewer.setLabelProvider(new DescriptiveLabelProvider());
		
		descriptionViewer.setAutoExpandLevel(2);
						
		// Propagate selection from viewer
		getSite().setSelectionProvider(descriptionViewer);
		
		// Add context menu to tree
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, descriptionViewer);
		
		Control control = descriptionViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);			
		
		showAllElementsAction = new ToggleDescriptionAction(false);		
		hideAllElementsAction = new ToggleDescriptionAction(true);
		
		createToolbar();
	}
	
	

	private void createToolbar() {
		IToolBarManager toolBarManager = getViewSite().getActionBars().getToolBarManager();
		toolBarManager.add(showAllElementsAction);
		toolBarManager.add(hideAllElementsAction);
	}
	
	private void createMenu(){
		IMenuManager menuManager = getViewSite().getActionBars().getMenuManager();
		menuManager.add(showAllElementsAction);
		menuManager.add(hideAllElementsAction);
	}

	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		if(EditorUtil.getActiveEditor() == null){
			showEmptyPage();
			return;
		}
		
		if(part instanceof BulkEditor){
			showEmptyPage();
			return;
		}
		
		if(part instanceof MultiPageTaxonEditor){
			if(! part.equals(this.part)){
				IEditorInput input = ((IEditorPart) part).getEditorInput();
				descriptionFeatureCache.clear();
				showViewer(part, new StructuredSelection(input));
			}
		}
	}
	
	/**
	 * 
	 * @author n.hoffmann
	 * @created May 28, 2010
	 * @version 1.0
	 */
	private class ToggleDescriptionAction extends Action{
		private boolean expanded;
		
		public ToggleDescriptionAction(boolean expand){
			super(null, Action.AS_PUSH_BUTTON);
			expanded = expand;
			setImageAndTooltip();
		}
		
		private void setImageAndTooltip(){
			setImageDescriptor(new ImageDescriptor() {
				@Override
				public ImageData getImageData() {
					String resource = expanded ? ImageResources.COLLAPSE_ALL : ImageResources.EXPAND_ALL;
					return ImageResources.getImage(resource).getImageData();
				}
			});
			
			String toolTipText = expanded ? "Collapse all" : "Show all descriptive data";
			setToolTipText(toolTipText);
		}
		
		@Override
		public void run() {
			if(expanded){
				descriptionViewer.collapseAll();
			}else{
				descriptionViewer.expandAll();
			}
			setImageAndTooltip();
		}
	}

	@Override
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		boolean result = super.postOperation(objectAffectedByOperation);
		
		StructuredSelection selection;
		if(objectAffectedByOperation == null){
			selection = new StructuredSelection();
		}else {
			selection = new StructuredSelection(objectAffectedByOperation);
		}
		descriptionViewer.setSelection(selection, true);
		
		
		return result;
	}
	
	@Override
	public void dirtyStateChanged() {
		descriptionViewer.refresh(true);
		super.dirtyStateChanged();
	}


	@Override
	public Viewer getViewer() {
		return descriptionViewer;
	}



	@Override
	protected Object getInitialInput() {
		return getEditor().getEditorInput();
	}



	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}
