// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.editor.view.detail;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnitBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.Page;
import eu.etaxonomy.taxeditor.editor.name.AbstractGroupedContainer;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmDataViewer;
import eu.etaxonomy.taxeditor.editor.view.AbstractCdmViewPart;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.DetailType;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory.EntityDetailType;
import eu.etaxonomy.taxeditor.forms.RootElement;

/**
 * @author n.hoffmann
 * @created Feb 12, 2010
 * @version 1.0
 */
public class DetailsViewer extends AbstractCdmDataViewer {
	
	private static final Logger logger = Logger
			.getLogger(DetailsViewer.class);

	private enum VIEW_PART{
		TAXON, 
		NAME, 
		REFEERENCE, 
		TEAM,
		PERSON,
		DESCRIPTION,
		DESCRIPTION_ELEMENT,
		EMPTY, 
		MEDIA, 
		TEAM_OR_PERSON_BASE,
		DERIVED_UNIT_BASE,
		FIELD_OBSERVATION
	}
	
	private VIEW_PART currentViewPart;

	private ISelection selection;

	private AbstractGroupedContainer sourceContainer;
	
	/**
	 * @param parent
	 */
	public DetailsViewer(Composite parent, AbstractCdmViewPart viewPart) {
		super(parent, viewPart);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#refresh()
	 */
	@Override
	protected void showParts() {	
		if(getInput() instanceof TaxonBase){	
			if(currentViewPart != VIEW_PART.TAXON){
				createTaxonSections(rootElement);
				currentViewPart = VIEW_PART.TAXON;
				
				// preserve a pointer to the AbstractNameComposite
				MultiPageTaxonEditor editor = EditorUtil.getActiveMultiPageTaxonEditor();
				if(editor != null){
					TaxonNameEditor page = (TaxonNameEditor) editor.getPage(Page.NAME);
					for(AbstractGroupedContainer container : page.getGroupedContainers()){
						if(container.getData() == getInput()){
							sourceContainer = container;
						}
					}
				}				
			}			
		}else if(getInput() instanceof NonViralName){
			if(currentViewPart != VIEW_PART.NAME){
				createNameSections(rootElement);
				currentViewPart = VIEW_PART.NAME;
			}
		}else if(getInput() instanceof ReferenceBase){
			if(currentViewPart != VIEW_PART.REFEERENCE){
				createReferenceSections(rootElement);
				currentViewPart = VIEW_PART.REFEERENCE;
			}
		}else if(getInput() instanceof Team){
			if(currentViewPart != VIEW_PART.TEAM){
				createTeamDetailSection(rootElement);
				currentViewPart = VIEW_PART.TEAM;
			}
		}else if(getInput() instanceof Person){
			if(currentViewPart != VIEW_PART.PERSON){
				createPersonDetailSection(rootElement);
				currentViewPart = VIEW_PART.PERSON;
			}
		}else if(getInput() instanceof TeamOrPersonBase){
			// fallback
			if(currentViewPart != VIEW_PART.TEAM_OR_PERSON_BASE){
				createTeamOrPersonBaseDetailSection(rootElement);
				currentViewPart = VIEW_PART.TEAM_OR_PERSON_BASE;
			}
		}else if(getInput() instanceof DescriptionBase){
			if(currentViewPart != VIEW_PART.DESCRIPTION){
				createDescriptionSection(rootElement);
				currentViewPart = VIEW_PART.DESCRIPTION;
			}
		}else if(getInput() instanceof DescriptionElementBase){
			if(currentViewPart != VIEW_PART.DESCRIPTION_ELEMENT){
				createDescriptionElementSection(rootElement);
				currentViewPart = VIEW_PART.DESCRIPTION_ELEMENT;
			}
		}else if(getInput() instanceof Media){
			if(currentViewPart != VIEW_PART.MEDIA){
				createMediaElementSection(rootElement);
				currentViewPart = VIEW_PART.MEDIA;
			}
		}else if(getInput() instanceof DerivedUnitBase){
			if(currentViewPart != VIEW_PART.DERIVED_UNIT_BASE){
				createDerivedUnitBaseElementSection(rootElement);
				currentViewPart = VIEW_PART.DERIVED_UNIT_BASE;
			}			
		}else if(getInput() instanceof DerivedUnitBase){
			if(currentViewPart != VIEW_PART.FIELD_OBSERVATION){
				createFieldObservationElementSection(rootElement);
				currentViewPart = VIEW_PART.FIELD_OBSERVATION;
			}			
		}else{
			destroySections();
			currentViewPart = VIEW_PART.EMPTY;
		}
		
		layout();
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#getSelection()
	 */
	@Override
	public ISelection getSelection() {
		return selection;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.viewers.Viewer#setSelection(org.eclipse.jface.viewers.ISelection, boolean)
	 */
	@Override
	public void setSelection(ISelection selection, boolean reveal) {
		this.selection = selection;
		SelectionChangedEvent selectionChangedEvent = new SelectionChangedEvent(this, selection);
		fireSelectionChanged(selectionChangedEvent);
	}
	
	
	private void createTaxonSections(RootElement parent) {
		destroySections();
		
		AbstractFormSection taxonBaseDetailSection = 
			formFactory.createCdmDetailSection(DetailType.TAXONBASE, getConversationHolder(), parent, this, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection nonViralNameSection = 
			formFactory.createCdmDetailSection(DetailType.NONVIRALNAME, getConversationHolder(), parent, this, Section.TWISTIE | Section.EXPANDED);
				
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection referenceBaseDetailSection = 
			 formFactory.createCdmDetailSection(DetailType.NOMENCLATURALREFERENCE, getConversationHolder(), parent, this, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection nomenclaturalStatusSection = 
			 formFactory.createEntityDetailSection(EntityDetailType.NOMENCLATURALSTATUS, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection protologSection = 
			 formFactory.createEntityDetailSection(EntityDetailType.PROTOLOG, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection typeDesignationSection = 
			formFactory.createEntityDetailSection(EntityDetailType.TYPEDESIGNATION, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection nameRelationshipSection = 
			formFactory.createEntityDetailSection(EntityDetailType.NAME_RELATIONSHIP, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection parsingMessagesSection = 
			 formFactory.createCdmDetailSection(DetailType.PARSINGMESSAGE, getConversationHolder(), parent, this, Section.EXPANDED);
		
		
		
		addPart(taxonBaseDetailSection);	
		addPart(nonViralNameSection);		
		addPart(nomenclaturalStatusSection);
		addPart(protologSection);		
		addPart(referenceBaseDetailSection);
		addPart(typeDesignationSection);
		addPart(parsingMessagesSection);
		addPart(nameRelationshipSection);
	}
	
	private void createNameSections(RootElement parent) {
		destroySections();
		AbstractFormSection nameDetailSection = 
			formFactory.createCdmDetailSection(DetailType.NONVIRALNAME, getConversationHolder(), parent, this, Section.EXPANDED);
		
		addPart(nameDetailSection);	
	}
	
	private void createReferenceSections(RootElement parent) {
		destroySections();
		AbstractFormSection referenceBaseDetailSection = 
			formFactory.createCdmDetailSection(DetailType.REFERENCEBASE, getConversationHolder(), parent, this, Section.EXPANDED);
		addPart(referenceBaseDetailSection);
	}
	
	private void createTeamOrPersonBaseDetailSection(RootElement parent) {
		destroySections();
		AbstractFormSection teamOrPersonBaseDetailSection = 
			formFactory.createCdmDetailSection(DetailType.TEAMORPERSONBASE, getConversationHolder(), parent, this, Section.EXPANDED);
		
		addPart(teamOrPersonBaseDetailSection);
	}
	
	private void createTeamDetailSection(RootElement parent){
		destroySections();
		AbstractFormSection teamDetailSection = 
			formFactory.createCdmDetailSection(DetailType.TEAM, getConversationHolder(), parent, this, Section.EXPANDED);
			
		addPart(teamDetailSection);
	}
	

	private void createPersonDetailSection(RootElement parent) {
		destroySections();
		AbstractFormSection personDetailSection = 
			formFactory.createCdmDetailSection(DetailType.PERSON, getConversationHolder(), parent, this, Section.EXPANDED);
		addPart(personDetailSection);
	}
	
	private void createDescriptionElementSection(RootElement parent) {
		destroySections();
		
		AbstractFormSection descriptionElementDetailSection =
			formFactory.createCdmDetailSection(DetailType.DESCRIPTIONELEMENT, getConversationHolder(), parent, this, Section.EXPANDED);

		AbstractFormSection descriptionElementSourceSection = 
			formFactory.createEntityDetailSection(EntityDetailType.DESCRIPTIONELEMENTSOURCE, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection descriptionElementMediaSection = 
			formFactory.createEntityDetailSection(EntityDetailType.DESCRIPTIONELEMENTMEDIA, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(descriptionElementDetailSection);
		addPart(descriptionElementSourceSection);
		addPart(descriptionElementMediaSection);
	}

	private void createDescriptionSection(RootElement parent) {
		destroySections();
		AbstractFormSection descriptionDetailSection = 
			formFactory.createCdmDetailSection(DetailType.DESCRIPTION, getConversationHolder(), parent, this, Section.EXPANDED);
		AbstractFormSection descriptionSourceSection =
			formFactory.createEntityDetailSection(EntityDetailType.DESCRIPTIONSOURCE, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		AbstractFormSection scopeSection = 
			formFactory.createEntityDetailSection(EntityDetailType.SCOPE, getConversationHolder(), parent, Section.TWISTIE);
		
		formFactory.createHorizontalSeparator(parent, SWT.BORDER);
		
		addPart(descriptionDetailSection);
		addPart(descriptionSourceSection);
		addPart(scopeSection);
	}
	

	private void createMediaElementSection(RootElement parent) {
		destroySections();
		AbstractFormSection mediaDetailSection = formFactory.createCdmDetailSection(DetailType.MEDIA, getConversationHolder(), parent, this, Section.EXPANDED); 
		
		addPart(mediaDetailSection);
	}
	
	private void createDerivedUnitBaseElementSection(RootElement parent){
		destroySections();
		
		AbstractFormSection derivedUnitBaseElementSection = formFactory.createCdmDetailSection(DetailType.DERIVED_UNIT_FACADE, getConversationHolder(), parent, this, Section.EXPANDED);
		
		addPart(derivedUnitBaseElementSection);
	}
	
	private void createFieldObservationElementSection(RootElement parent){
		destroySections();
		
		AbstractFormSection fieldObservationElementSection = formFactory.createCdmDetailSection(DetailType.FIELD_OBSERVATION, getConversationHolder(), parent, this, Section.EXPANDED);
		
		addPart(fieldObservationElementSection);
	}
}
