/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.Section;

/**
 * @author n.hoffmann
 *
 */
public abstract class AbstractCdmFormElement implements IPropertyChangeEmitter, ICdmFormElement, ISelectable {

	protected CdmFormFactory formFactory;

	private ListenerList propertyChangeListeners;

	private Composite layoutComposite;
	
	private Set<Control> controls = new HashSet<Control>();
	
	private Set<ICdmFormElement> elements = new HashSet<ICdmFormElement>();
	private ICdmFormElement parentElement;
	
	
	protected AbstractCdmFormElement(CdmFormFactory formFactory, Composite layoutComposite){
		this.layoutComposite = layoutComposite;
		this.formFactory = formFactory;
	}
	
	/**
	 * @param parent
	 * @param style
	 */
	public AbstractCdmFormElement(CdmFormFactory formFactory, ICdmFormElement formElement) {
		this(formFactory, formElement.getLayoutComposite());
		this.parentElement = formElement;
//		addControl(layoutComposite);
	}
	
	public CdmFormFactory getFormFactory() {
		return formFactory;
	}
	
	public Color getColor(boolean selected) {
		return selected ? SELECTED : NOT_SELECTED;
	}
	
	/**
	 * Returns all Controls that are managed by this element 
	 * @return
	 */
	public Set<Control> getControls(){
		return controls;
	}
	
	/**
	 * adds the control to the set of controls that are managed by this element
	 * @param child
	 */
	protected void addControl(Control child){
		controls.add(child);
	}
	
	protected void removeControl(Control child){
		controls.remove(child);
	}
	
	public Set<ICdmFormElement> getElements(){
		return elements;
	}
	
	public ICdmFormElement getParentElement(){
		return parentElement;
	}
	
	public void addElement(ICdmFormElement element){
		elements.add(element);
	}
	
	protected void removeElement(ICdmFormElement element){
		elements.remove(element);
	}
	
	public void removeElements(){
		for(ICdmFormElement childElement : getElements()){
			// recursion
			childElement.removeElements();
			
			// unregister selection arbitrator
			if(childElement instanceof ISelectableElement){
				SelectionArbitrator selectionArbitrator = ((ISelectableElement) childElement).getSelectionArbitrator();
				if(selectionArbitrator != null){
					formFactory.destroySelectionArbitrator(selectionArbitrator);
				}
			}
			
			// dispose of the controls
			
			removeControls(childElement);
			
//			for(Control control : childElement.getControls()){
//				// we added the layoutComposite of the parental element as the layout composite to this formElement
//				// but we do not want to destroy it.
//				if(control.equals(childElement.getLayoutComposite())){
//					continue;
//				}else{
//					control.dispose();
//					control = null;
//				}
//			}
		}
//		removeControls(this);
		elements.clear();
	}
	
	private void removeControls(ICdmFormElement element){
		if(element instanceof Section){
			((Section) element).dispose();
			element = null;
		}else{
			for(Control control : element.getControls()){
				// we added the layoutComposite of the parental element as the layout composite to this formElement
				// but we do not want to destroy it.
				if(control.equals(element.getLayoutComposite())){
					continue;
				}else{
					control.dispose();
					control = null;
				}
			}
		}
	}
	
	public Composite getLayoutComposite() {
		return layoutComposite;
	}
	
	public void setLayoutComposite(Composite layoutComposite){
		this.layoutComposite = layoutComposite;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#getPropertyChangeListeners()
	 */
	public ListenerList getPropertyChangeListeners() {
		return propertyChangeListeners;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#setPropertyChangeListeners(org.eclipse.core.runtime.ListenerList)
	 */
	public void setPropertyChangeListeners(ListenerList propertyChangeListeners){
		this.propertyChangeListeners = propertyChangeListeners;
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.IPropertyChangeEmitter#firePropertyChangeEvent()
	 */
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormComposite#firePropertyChangeEvent(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void firePropertyChangeEvent(CdmPropertyChangeEvent event) {
		Assert.isNotNull(propertyChangeListeners, "Property change listeners are not present");
		for(Object listener : propertyChangeListeners.getListeners()){
			((IPropertyChangeListener)listener).propertyChange(event);
		}
	}
	

	/**
	 * This method gets called whenever the toolkit this composite was created with gets a property change notification.
	 * 
	 * It is good advice to check whether the PropertyChangeEvent is destined for the implementing composite.
	 * Implementations should also check for null PropertyChangeEvents and return immediately in that case.
	 * 
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormElement#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent event) {
		// implement in subclasses
	}
	
	/**
	 * 
	 * @param formElement
	 * @return
	 */
	public boolean containsFormElement(ICdmFormElement formElement){
		if(formElement == this){
			return true;
		}else{
			for(ICdmFormElement element : getElements()){
				boolean contains = element.containsFormElement(formElement);
				if(contains == true){
					return true;
				}
			}
			return false;
		}
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.ICdmFormElement#refresh()
	 */
	public void refresh() {
		// empty default implementation	
	}
}
