/**
 * 
 */
package eu.etaxonomy.taxeditor.section;

import java.util.Collection;

import org.apache.commons.collections.CollectionUtils;
import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.VersionableEntity;
import eu.etaxonomy.taxeditor.forms.AbstractFormSection;
import eu.etaxonomy.taxeditor.forms.CdmFormFactory;
import eu.etaxonomy.taxeditor.forms.CdmPropertyChangeEvent;
import eu.etaxonomy.taxeditor.forms.ICdmFormElement;
import eu.etaxonomy.taxeditor.model.ImageResources;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 *
 */
public abstract class AbstractEntityCollectionSection<ENTITY extends VersionableEntity, ELEMENT extends VersionableEntity> extends AbstractFormSection<ENTITY> implements IExpansionListener{
	
	private static final Logger logger = Logger
				.getLogger(AbstractEntityCollectionSection.class);
	
	protected Composite container;
	
	private Label label_empty;

	private String title;
	
	/**
	 * @param conversation TODO
	 * @param parentElement
	 * @param style
	 */
	public AbstractEntityCollectionSection(CdmFormFactory formFactory, ConversationHolder conversation, ICdmFormElement parentElement, String title, int style) {
		super(formFactory, conversation, parentElement, Section.CLIENT_INDENT | style);
		this.title = title;
		this.setText(getTitleString());
		setTextClient(createToolbar());
		
		addExpansionListener(this);
		
		renderContent();
		
//		createDynamicContents();
	}
	
	private Control createToolbar() {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
		
		Action addAction = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				ELEMENT element = createNewElement();
				if(element != null){
					addElement(element);
					if(! getSection().isExpanded())
						getSection().setExpanded(true);
					internalUpdateSection(true);
				}
			}
		};
		addAction.setImageDescriptor(new ImageDescriptor() {
			
			@Override
			public ImageData getImageData() {
				return ImageResources.getImage(ImageResources.ADD_ICON).getImageData();
			}
		});
		addAction.setToolTipText(getTooltipString());
		
		toolBarManager.add(addAction);
		
		return toolBarManager.createControl(this);
	}
	
	public void setEntity(ENTITY entity) {
		if(entity != null && hasCollectionChanged(entity)){
			super.setEntity(entity);
			internalUpdateSection(false);
		}
		setSectionTitle();
		layout();
	};
		
	/**
	 * Sets the title for the section. Adds a "+" sign if the collection is not empty for this section.
	 * Override in subclasses if you want to have a different behaviour.
	 * 
	 */
	protected void setSectionTitle() {
		if(getCollection(getEntity()) != null && getCollection(getEntity()).size() > 0){
			this.setText(getTitleString() + " +");
		}else{
			this.setText(getTitleString());
		}
	}
	
	/**
	 * Removes all content from the container
	 */
	private void destroyDynamicContent(){
		if(label_empty != null){
			label_empty.dispose();
			label_empty = null;
		}
		removeElements();
	}
	
	/**
	 * Call this method after dynamically changing the client area. 
	 * If the options changed is set to true, will also fire a state changed
	 * event to inform the user of unsaved changes. 
	 * 
	 * @param changed 
	 */
	protected void internalUpdateSection(boolean changed){
		renderContent();
		if(changed)
			firePropertyChangeEvent(new CdmPropertyChangeEvent(this, null));
	}

	/**
	 * Whether the entities specific collection changed
	 * 
	 * @param newEntity
	 * @return
	 */
	private boolean hasCollectionChanged(ENTITY newEntity){

		// return true on null
		if(getEntity() == null || newEntity == null)	return true;	
		
		// if the entities differ the collection has changed
		if(! getEntity().equals(newEntity)) return true;
		
		Collection<ELEMENT> oldCollection = getCollection(getEntity());
		Collection<ELEMENT> newCollection = getCollection(newEntity);
		
		// return true on null
		if(oldCollection == null || newCollection == null) return true;
		
		// if the collections are object equal, check if the content is equal, too
		if(oldCollection.equals(newCollection)){
		
			boolean equal = CollectionUtils.isEqualCollection(oldCollection, newCollection);
//			
//			
//			boolean result = equal ? false : true;
//			
//			if("ProtologueSection".equals(this.getClass().getSimpleName()))
//				if(equal){
//					logger.warn(this.getClass().getSimpleName() + " : collections are equal");
//				}else{
//					logger.warn(this.getClass().getSimpleName() + " : collections are not equal");
//				}
//			
			// return true when collections are not equal
			return equal ? false : true;
		}
//		if("ProtologueSection".equals(this.getClass().getSimpleName()))
//			logger.warn(this.getClass().getSimpleName() + " : collections are not equal");
		return true;
	}

	private void renderContent(){
		if(this.isExpanded()){
//			logger.warn("rendering content");
			destroyDynamicContent();
			createDynamicContents();
		}
		this.reflow();
	}
	
	/**
	 * Creates the widgets for the collection
	 */
	protected void createDynamicContents(){
		
		Collection<ELEMENT> elements = null;
		if(getEntity() != null){
			elements = getCollection(getEntity());
		}
		
		if(elements == null || elements.size() == 0){
			label_empty = formFactory.createLabel(getLayoutComposite(), getEmptyString());
		}else{
			int i = 0;
			for(final ELEMENT element : elements){
				SelectionAdapter removeListener = new SelectionAdapter(){
					@Override
					public void widgetSelected(SelectionEvent e) {
						removeElement(element);
						internalUpdateSection(true);
					}
				};
				boolean modulo = i++%2 == 0;
				Color color = modulo ? getListBackgroundColorEven() : getListBackgroundColorOdd();
				createElementComposite(element, removeListener, color);
			}
		}
	}
		
	private Color getListBackgroundColorEven() {
		Color color = StoreUtil.getColor("eu.etaxonomy.taxeditor.preferences.colorDefinition.listBackgroundEven");
		return color;
	}

	private Color getListBackgroundColorOdd() {
		Color color = StoreUtil.getColor("eu.etaxonomy.taxeditor.preferences.colorDefinition.listBackgroundOdd");
		return color;
	}

	/**
	 * Create the specific widget for the element
	 * @param element
	 */
	protected void createElementComposite(ELEMENT element, SelectionListener removeListener, Color backgroundColor){
		AbstractEntityCollectionElement formElement = formFactory.createEntityCollectionElement(this, element, removeListener, backgroundColor, SWT.NULL);
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.section.AbstractEditorFormSection#setBackground(org.eclipse.swt.graphics.Color)
	 */
	@Override
	public void setBackground(Color color) {
		if(label_empty != null && !label_empty.isDisposed()){
			label_empty.setBackground(color);
		}
		super.setBackground(color);
	}

	/**
	 * Get the specific collection of this entity
	 * @param entity
	 * @return
	 */
	public abstract Collection<ELEMENT> getCollection(ENTITY entity);
	
	/**
	 * Create a new Element for this collection
	 * @return
	 */
	public abstract ELEMENT createNewElement();
	
	/**
	 * Add an element to the entities collection
	 * 
	 * @param element
	 */
	public abstract void addElement(ELEMENT element);
	
	/**
	 * Remove an element from the entities collection
	 * 
	 * @param element
	 */
	public abstract void removeElement(ELEMENT element);
	
	/**
	 * String to display when the collection is empty
	 * @return
	 */
	public abstract String getEmptyString();
	
	/**
	 * 
	 * @return String to display when hovering the add button
	 */
	protected abstract String getTooltipString();

	/**
	 * 
	 * @return 
	 */
	public String getTitleString() {
		return CdmUtils.Nz(title);
	}
	
	public void setTitleString(String title){
		this.title = title;
	}
	

	
	public void expansionStateChanging(ExpansionEvent e) {
//		logger.warn("Expansion State Changing");
	}
	
	public void expansionStateChanged(ExpansionEvent e) {
		renderContent();
	}
}
