
package eu.etaxonomy.taxeditor.navigation;

import java.io.File;
import java.io.FileNotFoundException;
import java.lang.reflect.InvocationTargetException;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IPersistable;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchListener;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.XMLMemento;

import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInput;
import eu.etaxonomy.taxeditor.editor.TaxonEditorInputFactory;
import eu.etaxonomy.taxeditor.editor.internal.TaxeditorEditorPlugin;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.MementoHelper;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * 
 * @author n.hoffmann
 * @created Apr 1, 2010
 * @version 1.0
 */
public class StateManager implements IContextListener {
	private static final Logger logger = Logger.getLogger(StateManager.class);
	
	private static final String EDITORS = "editors";

	private static final String EDITOR = "editor";
	
	private TaxonEditorInputFactory inputFactory;

	private IMemento memento;


	public StateManager(){
		inputFactory = new TaxonEditorInputFactory();
		PlatformUI.getWorkbench().addWorkbenchListener(new IWorkbenchListener() {

			public void postShutdown(IWorkbench workbench) {}

			public boolean preShutdown(IWorkbench workbench, boolean forced) {
				if(contextAboutToStop()){
					contextStop();
				}
				// return true in any case, otherwise the application will not stop
				return true;
			}
		});
	}
	
	/*
	 * (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.model.IContextListener#onContextEvent(eu.etaxonomy.taxeditor.model.IContextListener.EventType)
	 */
	public boolean onContextEvent(EventType eventType) {
		switch(eventType){
		case START:
			return contextStart();
		case STOP:
			return contextStop();
		case ABOUT_TO_STOP:
			return contextAboutToStop();
		}
		return false;
	}
	
	/**
	 * Tries to restore the state of the taxon editor and the navigator when a context starts.
	 * 
	 * @return true in any case
	 */
	private boolean contextStart() {
		IMemento memento = null;
		try {
			memento = MementoHelper.readMementoFromFile(getStateFileForCurrentDatabase());
		} catch (FileNotFoundException e) {
			// no memento -> no previous state
		}
		
		

		restoreNavigator(memento);
		
		restoreEditors(memento); 
		
		
		return true;
	}
	
	/**
	 * Reads taxon node UUIDs from the given memento and tries to open an editor 
	 * for each of the found UUIDs.
	 * 
	 * @param memento
	 */
	private void restoreEditors(final IMemento memento){
		if(memento == null){
			return;
		}
		
		IRunnableWithProgress runnable = new IRunnableWithProgress() {
			
			public void run(IProgressMonitor monitor) throws InvocationTargetException,
					InterruptedException {
				IMemento editorMementos = memento.getChild(EDITORS);
				IMemento[] editorInputs = editorMementos.getChildren(EDITOR);
				
				monitor.beginTask("Restoring Editors", 1 + editorInputs.length * 2);
				monitor.worked(1);
				
				for(IMemento editorInput : editorInputs){
					TaxonEditorInput input = (TaxonEditorInput) inputFactory.createElement(editorInput);
					monitor.worked(1);
					if(!monitor.isCanceled() && input != null){
						try {
							EditorUtil.open(input);
						} catch (PartInitException e) {
							logger.error("Error opening an editor window", e);
						}
					}
					monitor.worked(1);
				}
				monitor.done();
			}
		};
		
		try {
			NavigationUtil.run(false, true, runnable);
		} catch (InvocationTargetException e) {
			logger.error("InvocationTargetException when trying to restore open editors", e);
		} catch (InterruptedException e) {
			logger.error("InterruptedException when trying to restore open editors", e);
		}
	}
	
	/**
	 * Restores the state of the navigator. Will open all previously opened paths.
	 * 
	 * @param memento
	 */
	private void restoreNavigator(IMemento memento){
		TaxonNavigator navigator = NavigationUtil.getNavigator(true);
		navigator.restore(memento);
	}
	
	/**
	 * Saves editor state to memento when the the context is about to shut down.
	 * The memento gets created anew on every shutdown process as to get rid of
	 * stale data from previous sessions on this context.
	 * 
	 * After this point the user may still cancel the process.
	 * 
	 * @return
	 */
	private boolean contextAboutToStop() {
		// we are creating the memento here; even if the context is not stopped
		if (CdmStore.getDataSource() != null) {
			
			try {
				memento = XMLMemento.createWriteRoot(CdmStore.getDataSource().getName());
			
				saveEditorState(memento);
				logger.warn("DataSource found. Memento created.");
			} catch (Exception e) {
				// The memento could not be created, but a not closable editor is avoided for this case.
				logger.error("The memento could not be created: " + e.getMessage());
				return false;
			}
		} else {
			logger.error("No DataSource present.");
			return false;
		}
		
		return true;
	}
	
	/**
	 * If the context 
	 * 
	 * @return
	 */
	private boolean contextStop() {
		saveNavigatorState(memento);
		
		// save the memento when the context really stops
		return MementoHelper.saveMementoToFile(memento, getStateFileForCurrentDatabase()) != null;
	}
	
	/**
	 * Saves the UUIDs 
	 * 
	 * @param memento
	 */
	private void saveEditorState(IMemento memento){
		Set<IEditorPart> editors = EditorUtil.getOpenEditors();
		
		IMemento editorMementos = memento.createChild(EDITORS);
		
		for(IEditorPart editor : editors){
			IEditorInput input = editor.getEditorInput();
			if(input instanceof IPersistable){
				IMemento editorMemento = editorMementos.createChild(EDITOR);
				((IPersistable) input).saveState(editorMemento);
			}
		}
	}
	
	/**
	 * 
	 * @param memento
	 */
	private void saveNavigatorState(IMemento memento){
		TaxonNavigator navigator = NavigationUtil.getNavigator(false);
		if(navigator != null){
			navigator.save(memento);
		}
	}

	/**
	 * 
	 * @return
	 */
	private File getStateFileForCurrentDatabase() {
		IPath path = TaxeditorEditorPlugin.getDefault().getStateLocation();
		if (path == null) {
			return null;
		}
		path = path.append("editor_state_" + CdmStore.getDataSource().getName() + ".xml");
		return path.toFile();
	}

}
