// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.Command;
import org.eclipse.core.commands.common.NotDefinedException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.commands.ICommandService;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.navigator.CommonNavigator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IDataChangeBehavior;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/** 
 * Taxonomic tree implementation using Common Navigator Framework.
 * 
 * @author p.ciardelli
 * @created 02.06.2009
 * @version 1.0
 */
public class TaxonNavigator extends CommonNavigator implements IPostOperationEnabled, IConversationEnabled{
	private static final Logger logger = Logger
			.getLogger(TaxonNavigator.class);

	public static final String ID = "eu.etaxonomy.taxeditor.navigation.navigator"; //$NON-NLS-1$

	public static final String OPEN_COMMAND_ID = "eu.etaxonomy.taxeditor.navigation.command.editSelection";
	
	private static final String TREE_PATH = "treepath";

	private static final String TREE_PATHS = "treepaths";

	private ConversationHolder conversation;

	private IDataChangeBehavior dataChangeBehavior;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#getInitialInput()
	 */
	@Override
	protected IAdaptable getInitialInput() {
		if(CdmStore.isActive()){
			// TODO when closing and reopening the taxon navigator 
			// we do not preserve state. Closing the view, in contrary to closing the whole application
			// should be handled by the state manager too
						
			return new Root(conversation);
		}
		return new EmptyRoot();
	}
	
	/**
	 * 
	 */
	public void init(IViewSite site)
			throws PartInitException {
		super.init(site);
		if(CdmStore.isActive() && conversation == null){
			conversation = CdmStore.createConversation();
			conversation.registerForDataStoreChanges(TaxonNavigator.this);
		}
	}
	
	/**
	 * Refresh this navigators viewer
	 */
	public void refresh(){
		logger.warn("Refreshing navigator");
		getConversationHolder().bind();
		this.getCommonViewer().refresh();
	}

	/**
	 * 
	 * @param memento
	 */
	public void restore(final IMemento memento) {
		
		IRunnableWithProgress runnable = new IRunnableWithProgress() {
			
			public void run(IProgressMonitor monitor) throws InvocationTargetException,
					InterruptedException {
				monitor.setTaskName("Restoring Taxon Navigator");
				int mementoWork = 0;
				Set<TreePath> treePaths = new HashSet<TreePath>();
				IMemento[] treePathMementos = null;
				if (memento != null) {
					
					
					IMemento treePathsMemento = memento.getChild(TREE_PATHS);
					
					if (treePathsMemento != null) {
						treePathMementos = treePathsMemento.getChildren(TREE_PATH);
						mementoWork = treePathMementos.length;
					}
				}
				// begin the monitor with steps for all tree paths and steps for creating 
				// conversation s.o., refreshing the tree and setting the paths 
				monitor.beginTask("Restoring Taxon Navigator", 1 + mementoWork + 5);
				monitor.worked(1);
				
				conversation = CdmStore.createConversation();
				monitor.worked(1);
				conversation.registerForDataStoreChanges(TaxonNavigator.this);
				monitor.worked(1);
				TaxonNavigator.this.getCommonViewer().setInput(new Root(conversation));
				monitor.worked(1);
				TaxonNavigator.this.getCommonViewer().refresh();
				monitor.worked(1);
				
				
				
				if (treePathMementos != null && treePathMementos.length > 0) {
					for (IMemento treePathMemento : treePathMementos) {
						TreePath treePath = createTreePathFromString(treePathMemento.getID());
						if (!monitor.isCanceled() && treePath != null) {
							treePaths.add(treePath);
							monitor.worked(1);
						}
					}
				}
				if (treePaths.size() > 0) {
					TaxonNavigator.this.getCommonViewer().setExpandedTreePaths(treePaths.toArray(new TreePath[0]));
					monitor.worked(1);
				}
				monitor.done();
			}
		};
		
		try {
			NavigationUtil.run(false, true, runnable);
		} catch (InvocationTargetException e) {
			logger.error("InvocationTargetException when trying to restore navigator", e);
		} catch (InterruptedException e) {
			logger.error("InterruptedException when trying to restore navigator", e);
		}
	}

	/**
	 * @param string
	 * @return
	 */
	private TreePath createTreePathFromString(String string) {

		List<CdmBase> pathList = new ArrayList<CdmBase>();
		
		if(string.length() == 0) return null; 
		
		for (String uuid : string.split(" ")) {
			CdmBase cdmBaseObject = CdmStore.getTaxonTreeService().getTaxonNodeByUuid(UUID.fromString(uuid));
			if (cdmBaseObject == null) {
				// is this a tree uuid?
				cdmBaseObject = CdmStore.getTaxonTreeService().getTaxonomicTreeByUuid(UUID.fromString(uuid));
				
				if(cdmBaseObject == null) return null;
			}
			pathList.add(cdmBaseObject);
		}
		return new TreePath(pathList.toArray());
	}


	@Override
	public void saveState(IMemento aMemento) {
		//
	}
	
	public void saveTreeState(IMemento memento) {

		if (memento != null) {
			super.saveState(memento);
			
			memento = memento.createChild(TREE_PATHS);
			TreePath[] treePaths = this.getCommonViewer().getExpandedTreePaths();
			for (TreePath treePath : treePaths) {
				int pathLength = treePath.getSegmentCount();
				String path = "";
				for (int i = 0; i < pathLength; i++) {
					Object segment = treePath.getSegment(i);
					if (segment instanceof CdmBase) {
						path += ((CdmBase) segment).getUuid().toString() + " ";
					} else {
						logger.warn("Non-taxon tree path segment " + segment);
					}
				}
				logger.info("Creating memento child for path " + path.trim());
				memento.createChild(TREE_PATH, path.trim());
			}
		} else {
			logger.warn("TreeState could not be saved: Memento is null");
		}
	}
	
	/**
	 * Activates the current view.
	 */
	private void activateView() {
		   getSite().getPage().activate(this);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite aParent) {
		super.createPartControl(aParent);
		
		// It seems the workbench does not set the focus at the right time. This should fix this.
//		getCommonViewer().addSelectionChangedListener(new ISelectionChangedListener() {
//		   public void selectionChanged(SelectionChangedEvent event) {
//			     activateView();
//		   }
//		});
	}
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	public void update(CdmDataChangeMap changeEvents) {
		if(dataChangeBehavior == null){
			dataChangeBehavior = new TaxonNavigatorDataChangeBehavior(this);
		}
		
		DataChangeBridge.handleDataChange(changeEvents, dataChangeBehavior);
	}
	
	public String getFrameToolTipText(Object element) {
		if (element instanceof Root) {
			return "Taxonomic Tree";
		}
		return super.getFrameToolTipText(element);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		super.dispose();
		dataChangeBehavior = null;
		if(conversation != null){
			conversation.unregisterForDataStoreChanges(this);
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#setFocus()
	 */
	@Override
	public void setFocus() {
		//logger.warn("Setting focus to navigator");
		super.setFocus();
		if(getConversationHolder() != null){
			getConversationHolder().bind();
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		// nothing to do here
		return true;
	}

	public void save(IMemento memento) {
		saveTreeState(memento);
		if(conversation != null){
			conversation.unregisterForDataStoreChanges(this);
			conversation = null;
		}
//		this.getCommonViewer().setInput(new EmptyRoot());
	}

//	@Override
//	public ISelection getSelection() {
//		return getCommonViewer().getSelection();
//	}
	
	/**
	 * @see http://www.eclipse.org/forums/index.php?t=msg&goto=512094&
	 */
	@Override
	protected void handleDoubleClick(DoubleClickEvent anEvent) {
		
		ICommandService commandService = (ICommandService)getSite().getService(ICommandService.class);
		
		Command command = commandService.getCommand(OPEN_COMMAND_ID); 
		if(command.isEnabled()) {
			IHandlerService handlerService = (IHandlerService)getSite().getService(IHandlerService.class);
			try {
				handlerService.executeCommand(OPEN_COMMAND_ID, null);
			} catch (NotDefinedException e) {
				throw new RuntimeException("Could not find open command: " + OPEN_COMMAND_ID);
			} catch (Exception e) {
				logger.error("An exception occured while trying to open a selection", e);
			}
		}
		// If the double click is passed up to the super-class it will expand/collapse trees.
		// We do not want that
		//super.handleDoubleClick(anEvent);
	}

	public boolean onComplete() {
		// TODO Auto-generated method stub
		return false;
	}
}