// $Id$
/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.navigation.search;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.menus.WorkbenchWindowControlContribution;
import org.eclipse.ui.swt.IFocusService;

import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.internal.TaxeditorNavigationPlugin;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.preference.Resources;

/**
 * @author n.hoffmann
 * @author e.-m.lee
 * @created 15.04.2009
 * @version 1.0
 */
public class SearchBar extends WorkbenchWindowControlContribution{
	private static final Logger logger = Logger.getLogger(SearchBar.class);
	private Text text_search;
	private String srv;

	private String defaultText = "Use \"*\" for wildcard searching";

	final private ConfigurationSelectionListener configurationListener = new ConfigurationSelectionListener();

	@Override
	protected Control createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NONE);

		createLayout(composite);
		createSearchTextField(composite);
		createToolBar(composite);
		registerAtFocusService();

		return composite;
	}

	/**
	 * Handles focus changes for the search textfield.
	 */
	private void registerAtFocusService() {
		IFocusService focusService = 
			(IFocusService) PlatformUI.getWorkbench().getService(IFocusService.class);
		if (focusService != null) {
			focusService.addFocusTracker(text_search, "navigation.textControlId");
		}
	}

	/**
	 * Creates the search toolbar.
	 * @param composite
	 */
	private void createToolBar(Composite composite) {
		final ToolBar toolBar = new ToolBar(composite, SWT.NULL);

		ToolItem toolItem = new ToolItem(toolBar, SWT.DROP_DOWN | SWT.BORDER);
		toolItem.setText("Search");

		DropdownSelectionListener dropdownListener = new DropdownSelectionListener(
				toolItem);

		for(SearchOption searchOption : SearchOption.values()){
			dropdownListener.add(searchOption);
		}

		toolItem.addSelectionListener(dropdownListener);
	}

	/**
	 * Creates the search textfield.
	 * @param composite
	 */
	private void createSearchTextField(Composite composite) {
		// TODO for some reason the text_search composite has a margin when
		// either SWT.BORDER or SWT.SEARCH
		// is applied. I am not sure how to get rid of this.
		text_search = new Text(composite, SWT.BORDER | SWT.SINGLE
				| SWT.FULL_SELECTION);
		text_search.setForeground(NavigationUtil.getColor(Resources.SEARCH_VIEW_FOREGROUND));
		text_search.setText(defaultText);

		addTextListeners();
	}

	/**
	 * Adds listeners to the search textfield.
	 */
	private void addTextListeners() {
		text_search.addFocusListener(new FocusListener() {

			public void focusGained(FocusEvent e) {
				text_search.setForeground(NavigationUtil.getColor(Resources.SEARCH_VIEW_FOCUS));
				if (defaultText.equals(text_search.getText())) {
					text_search.setText("");
				}
			}

			public void focusLost(FocusEvent e) {
				if (text_search.getText() == "") {
					text_search.setForeground(NavigationUtil.getColor(Resources.SEARCH_VIEW_FOREGROUND));
					text_search.setText(defaultText);
				}
			}
		});

		text_search.addKeyListener(new KeyAdapter() {
			/*
			 * (non-Javadoc)
			 * @see org.eclipse.swt.events.KeyAdapter#keyPressed(org.eclipse.swt.events.KeyEvent)
			 */
			@Override
			public void keyPressed(KeyEvent e) {
				if (e.keyCode == SWT.CR) {
					search();
				}
			}
		});
	}

	/**
	 * Creates the search layout.
	 * @param composite
	 */
	private void createLayout(Composite composite) {
		final RowLayout layout = new RowLayout();
		layout.wrap = false;
		layout.pack = true;
		layout.justify = true;
		layout.type = SWT.HORIZONTAL;
		layout.marginLeft = 0;
		layout.marginTop = 0;
		layout.marginRight = 0;
		layout.marginBottom = 0;
		layout.spacing = 0;
		composite.setLayout(layout);
	}

	private void search(){
		final String searchString = getSearchString();
		if(searchString == null){
			return;
		}
		if("*".equals(searchString.trim())){
			NavigationUtil.warningDialog("Could not execute search", "Please type at least one character when using the \"*\" wildcard.");
			return;
		}
		
		ITaxonServiceConfigurator configurator = configurationListener.getConfigurator();
		configurator.setSearchString(searchString);
		openSearchResultsView(configurator);
		
	}
	
	private String getSearchString(){
		String searchString = text_search.getText().trim();
		if (searchString.equals(defaultText) || searchString.length() == 0)
			return null;
		return searchString;
	}
	
	/**
	 * Opens a new instance of the search result view to display the result to the user.
	 * 
	 * @param searchResult
	 */
	private void openSearchResultsView(ITaxonServiceConfigurator configurator) {
		// This variable is needed to address new instances of the view.
		srv += "1";
		logger.info("Opening search results window " + srv);
		try {
			IViewPart resultsView = TaxeditorNavigationPlugin.getDefault()
					.getWorkbench().getActiveWorkbenchWindow()
					.getActivePage().showView(SearchResultView.ID, srv,
							IWorkbenchPage.VIEW_ACTIVATE);
			((SearchResultView) resultsView).performSearch(configurator);
		} catch (PartInitException e) {
			logger.error("Error opening search result.", e);
			throw new RuntimeException(e);
		}
	}

	/**
	 * Handles drop down menu selection. Available items are defined in the enumeration SearchOption.
	 * 
	 * @author n.hoffmann
	 * @created Feb 2, 2010
	 * @version 1.0
	 */
	class DropdownSelectionListener extends SelectionAdapter {

		private Menu menu;

		public DropdownSelectionListener(ToolItem dropdown) {
			menu = new Menu(dropdown.getParent().getShell());
		}

		public void add(SearchOption option) {
			MenuItem menuItem = new MenuItem(menu, SWT.CHECK);
			menuItem.setData(option);
			menuItem.setText(option.getLabel());
			menuItem.setSelection(option.getPreference());
			menuItem.addSelectionListener(configurationListener);
		}

		public void widgetSelected(SelectionEvent event) {
			if (event.detail == SWT.ARROW) {
				ToolItem item = (ToolItem) event.widget;
				Rectangle rect = item.getBounds();
				Point pt = item.getParent().toDisplay(new Point(rect.x, rect.y));
				menu.setLocation(pt.x, pt.y + rect.height);
				menu.setVisible(true);
			} else {
				search();
			}
		}
	}

	/**
	 * Handles search configuration selection.
	 * 
	 * @author n.hoffmann
	 * @created Feb 2, 2010
	 * @version 1.0
	 */
	class ConfigurationSelectionListener extends SelectionAdapter {

		private ITaxonServiceConfigurator configurator = PreferencesUtil.getSearchConfigurator();
		
		/*
		 * (non-Javadoc)
		 * 
		 * @see
		 * org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse
		 * .swt.events.SelectionEvent)
		 */
		public void widgetSelected(SelectionEvent e) {
			logger.warn("configuration menu clicked");
			SearchOption option = (SearchOption) e.widget.getData();

			switch (option){
			case TAXON:
				configurator.setDoTaxa(configurator.isDoTaxa() ? false : true);
				break;
			case SYNONYM:
				configurator.setDoSynonyms(configurator.isDoSynonyms() ? false : true);
				break;
			case NAME:
				configurator.setDoNamesWithoutTaxa(configurator.isDoNamesWithoutTaxa() ? false : true);
				break;
			case COMMON_NAME:
				configurator.setDoTaxaByCommonNames(getConfigurator().isDoTaxaByCommonNames() ? false : true);
				break;				
			}
			
			saveConfigurator();
		}

		public ITaxonServiceConfigurator getConfigurator() {
			return configurator;
		}

		private void saveConfigurator() {
			PreferencesUtil.setSearchConfigurator(getConfigurator());
			this.configurator = PreferencesUtil.getSearchConfigurator();
		}
	}

	/**
	 * Available search options.
	 * 
	 * @author n.hoffmann
	 * @created Feb 2, 2010
	 * @version 1.0
	 */
	enum SearchOption {
		TAXON("Taxa"), SYNONYM("Synonyms"), NAME("Names (without taxa)"), 
		COMMON_NAME("Common Names");

		private String label;

		private SearchOption(String label) {
			this.label = label;
		}

		public String getLabel() {
			return label;
		}

		public boolean getPreference() {
			if (!PreferencesUtil.getPreferenceStore().contains(
					PreferencesUtil.TAXON_SERVICE_CONFIGURATOR_TAXA)) {
				// initializes the search configurator
				PreferencesUtil.initializeSearchConfigurator();
			}

			switch (this) {
			case TAXON:
				boolean result = PreferencesUtil.getPreferenceStore().getBoolean(
								PreferencesUtil.TAXON_SERVICE_CONFIGURATOR_TAXA);
				return result;
			case SYNONYM:
				return PreferencesUtil.getPreferenceStore().getBoolean(
						PreferencesUtil.TAXON_SERVICE_CONFIGURATOR_SYNONYMS);
			case NAME:
				return PreferencesUtil.getPreferenceStore().getBoolean(
						PreferencesUtil.TAXON_SERVICE_CONFIGURATOR_NAMES);
			case COMMON_NAME:
				return PreferencesUtil.getPreferenceStore().getBoolean(
								PreferencesUtil.TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES);
			}

			return true;
		}

	}
}
