// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.IIdentifiableEntity;
import eu.etaxonomy.cdm.model.common.ISourceable;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.LanguageStringBase;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.OriginalSourceBase;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.location.NamedArea;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author p.ciardelli
 * @created 02.04.2009
 * @version 1.0
 */
public class DescriptionHelper {
	private static final Logger logger = Logger
			.getLogger(DescriptionHelper.class);

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param language
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element, 
			Language language) {
		String cache = null;
		if (element instanceof TextData) {
			cache = ((TextData) element).getText(language);
		}
		if (element instanceof CommonTaxonName) {
			cache = ((CommonTaxonName) element).getName();
		}
		if (element instanceof TaxonInteraction) {
			Taxon taxon2 = ((TaxonInteraction) element).getTaxon2();
			if(taxon2 != null && taxon2.getName() != null){
				cache = taxon2.getName().getTitleCache();
			}else{
				cache = "No taxon chosen";
			}
			
		}
		if (element instanceof Distribution) {
			Distribution distribution = (Distribution) element;
			
			NamedArea area = distribution.getArea();
			if(area != null){
				cache =  area.getLabel();
				
				PresenceAbsenceTermBase<?> status = distribution.getStatus();
				if (status == null){
					cache += ", no status";
				}else {
					cache += ", " + status.getLabel();
				}
			}
		}
		return cache == null ? "" : cache;
	}

	/**
	 * Returns whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @return 
	 */
	public static String getCache(DescriptionElementBase element) {
		return getCache(element, CdmStore.getDefaultLanguage());
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class.
	 * 
	 * @param element
	 * @param value
	 * @param language
	 */
	public static void setCache(DescriptionElementBase element,
			String value, Language language) {
		if (element instanceof TextData) {
			((TextData) element).putText(value, language);
			return;
		}
		if (element instanceof CommonTaxonName) {
			((CommonTaxonName) element).setName(value);
			return;
		}
		if (element instanceof TaxonInteraction) {
			
		}
		if(element instanceof Distribution){
			logger.warn("trying to set cache on distribution, don't know what to do at the moment.");
			return;
		}
		logger.warn("No matching subclass found for DescriptionElementBase object, 'cache' not set.");
	}
	
	/**
	 * Set whatever the element's title cache equivalent is,
	 * depending on its class, using the default language.
	 * 
	 * @param element
	 * @param value
	 */	
	public static void setCache(DescriptionElementBase element,
			String value) {
		setCache(element, value, CdmStore.getDefaultLanguage());
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.bulkeditor.referencingobjects.IReferencingObjectsService#getObjectDescription(java.lang.Object)
	 */
	public static String getObjectDescription(Object element) {
		if (element instanceof IdentifiableEntity) {
			try{
				return ((IdentifiableEntity) element).getTitleCache();
			}catch(Exception e){
				// TODO we have to make sure that we will never get here. 
				// It is not clear at the moment how we can accomplish that.
				String result = "Error initializing title cache for object";
				logger.error(result);
				return "TODO" + result;
			}
		}
		if (element instanceof OriginalSourceBase) {
			OriginalSourceBase originalSource = (OriginalSourceBase) element;
//			ReferenceBase citation = HibernateProxyHelper.deproxy(originalSource.getCitation(), ReferenceBase.class);
			ISourceable sourcedObject = originalSource.getSourcedObj();
			String sourceObjectTitle = "";
			if(sourcedObject instanceof IIdentifiableEntity){
				sourceObjectTitle = ((IdentifiableEntity) sourcedObject).getTitleCache();
			}else if(sourcedObject instanceof DescriptionElementBase){
				sourceObjectTitle = "Element for description: " + ((DescriptionElementBase) sourcedObject).getInDescription().getTitleCache();
			}else{
				throw new IllegalStateException("Unknown ISourceable object for given OriginalSourceBase");
			}
			
			return CdmUtils.concat(";", new String[]{originalSource.getIdNamespace(), originalSource.getIdInSource(), sourceObjectTitle});
		}
		if (element instanceof LanguageStringBase) {
			return ((LanguageStringBase) element).getText();
		}
		if (element instanceof DescriptionElementBase) {
			return getCache((DescriptionElementBase) element);		
		}
		if (element instanceof Marker) {
			Marker marker = (Marker) element;
			return marker.getMarkerType().getLabel() + " (" + marker.getFlag() + ")";
		}
		// TODO write return texts for NameRelationship, HomotypicalGroup, SpecimenTypeDesignation, etc.
		return element.toString();
	}
}
