/**
 * 
 */
package eu.etaxonomy.taxeditor.parser;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.reference.INomenclaturalReference;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.parser.NonViralNameParserImpl;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 *
 */
public class ParseHandler{
	private static final Logger logger = Logger.getLogger(ParseHandler.class);

	private TaxonNameBase taxonNameBase;
	
	private List<INomenclaturalReference> duplicateReferences;
	private List<INomenclaturalReference> duplicateInReferences;

	private List<TaxonNameBase> duplicateNames;

	private List<TeamOrPersonBase> duplicateCombinationAuthorTeams;
	private List<TeamOrPersonBase> duplicateExCombinationAuthorTeams;
	private List<TeamOrPersonBase> duplicateBasionymAuthorTeams;
	private List<TeamOrPersonBase> duplicateExBasionymAuthorTeams;

	private static NonViralNameParserImpl nonViralNameParser = NonViralNameParserImpl.NewInstance();

	/**
	 * The name that should get parsed
	 */
	private NonViralName name;

	/**
	 * The widget, the name got entered in
	 */
	private Control textWidget;

	private boolean doResolveInReferences;
	
	/**
	 * Creates a new instance 
	 * 
	 * @param textWidget
	 * @param name
	 */
	private ParseHandler(Control textWidget, TaxonNameBase name){
		if(textWidget != null){
			this.textWidget = textWidget;
			checkControlHasText();
		}else{
			throw new IllegalArgumentException("text widget must not be null");
		}
		
		if(name == null){
			this.name = nonViralNameParser.getNonViralNameInstance("", PreferencesUtil.getPreferredNomenclaturalCode());
		}else{
			this.name = (NonViralName) HibernateProxyHelper.deproxy(name);
		}
	}
	
	/**
	 * Factory method to create a new instance of the this class
	 * 
	 * @param textWidget	The text widget where the NonViralName may be entered into
	 * @param name			An initial NonViralName or null when creating a new name 
	 * @return				An instance of this class 
	 */
	public static ParseHandler NewInstance(Control textWidget, TaxonNameBase name){
		return new ParseHandler(textWidget, name);
		
	}
	
	/**
	 * Parses a given string and returns a <code>TaxonNameBase</code> instance with the 
	 * reuslts of the parsing.
	 * 
	 * This method should be used to quickly create a new name from a string.
	 * Wherever the string will be parsed again in subsequent editing, an instance 
	 * of <code>ParseHandler</code> should be attached to the text widget.
	 * 
	 * @param unparsedNameString
	 * @return
	 */
	public static TaxonNameBase quickParse(String unparsedNameString){
		TaxonNameBase name = nonViralNameParser.parseReferencedName(unparsedNameString,
				PreferencesUtil.getPreferredNomenclaturalCode(), null);

		if (name.hasProblem()) {
			name.setFullTitleCache(unparsedNameString);
		}
		
		return name;
	}
	
	/**
	 * Creates an empty <code>TaxonNameBase</code> instance with the nomenclatural code
	 * currently set in preferences.
	 * 
	 * @return
	 */
	public static TaxonNameBase createEmptyName(){
		return nonViralNameParser.getNonViralNameInstance("", PreferencesUtil.getPreferredNomenclaturalCode());
	}
	
	/**
	 * Check if the given control has a getText() method
	 */
	private void checkControlHasText(){
		Class clazz = textWidget.getClass();
		
		try {
			clazz.getDeclaredMethod("getText", null);
		} catch (SecurityException e) {
		} catch (NoSuchMethodException e) {	
			throw new IllegalArgumentException("Given composite does not have a getText method");
		}
	}
	
	/**
	 * Parses the string that was entered into the text widget and returns a
	 * NonViralName object that resulted from the parsing process. 
	 * 
	 * @return	The parsed NonViralName object
	 */
	public NonViralName parse(){
		
		
		String unparsedNameString = "";
		try {
			Method getText;
			getText = textWidget.getClass().getDeclaredMethod("getText", null);
			unparsedNameString = (String) getText.invoke(textWidget, null);
		} catch (Exception e) {
			// we should never get here
			logger.error("Error trying to invoke getText method");
		}
		
		
		nonViralNameParser.parseReferencedName(name, unparsedNameString,
				name.getRank(), true);

		if (name.hasProblem()) {
			name.setFullTitleCache(unparsedNameString);
		}
		
		return name;
	}
	
	/**
	 * Parses the string that was entered into the text widget and returns a
	 * NonViralName object that resulted from the parsing process. 
	 * 
	 * The atomized fields (scientific name, author and reference) will be matched
	 * against the database to find possible duplicates. If duplicates were found 
	 * the respective parts of the NonViralName will be replaced with the found
	 * objects. 
	 *
	 * @return	The parsed NonViralName object
	 */
	public NonViralName parseAndResolveDuplicates(){
		
		parse();
		
		findMatches(name);
		
		resolveDuplicates(name);
		
		return name;
	}
		
	
	
	
	/** 
	 * @param name The name to resolve duplicates for.
	 */
	private void resolveDuplicates(NonViralName name) {
		resolveDuplicateNames(name);
		
		resolveAllDuplicateAuthors(name);
		
		resolveDuplicateReferences(name);
		
		if(duplicateInReferences != null)
			resolveDuplicateInReferences(name);
	}


	/** 
	 * @param name The name to resolve duplicates for.
	 */
	private void resolveDuplicateNames(NonViralName name) {
				
		if (duplicateNames.size() == 1){
			name = (NonViralName) duplicateNames.iterator().next();
		}else if(duplicateNames.size() > 1){
			// FIXME TODO resolve multiple duplications. Use first match for a start
			name = (NonViralName) duplicateNames.iterator().next();
		}
	}

	/** 
	 * @param name The name to resolve duplicates for.
	 */
	private void resolveDuplicateReferences(NonViralName name) {
		if(duplicateReferences.size() == 1){
			// exactly one match. We assume that the user wants this reference
			INomenclaturalReference duplicate = duplicateReferences.iterator().next();	
			name.setNomenclaturalReference(duplicate);
		}else if(duplicateReferences.size() > 1){
			// FIXME TODO resolve multiple duplications. Use first match for a start
			INomenclaturalReference duplicate = duplicateReferences.iterator().next();
			name.setNomenclaturalReference(duplicate);
		}		
	}

	/** 
	 * @param name The name to resolve duplicates for.
	 */
	private void resolveDuplicateInReferences(NonViralName name) {
		ReferenceBase reference = (ReferenceBase) HibernateProxyHelper.deproxy(name.getNomenclaturalReference());
		
		if(duplicateInReferences.size() > 0){
			ReferenceBase inReference = (ReferenceBase) duplicateInReferences.iterator().next();
			reference.setInReference(inReference);
			logger.warn(reference.generateTitle());
			// FIXME TODO resolve multiple duplications. We use first match for a start
			logger.warn(reference.getTitleCache());
		}
	}
	

	/** 
	 * @param name The name to resolve duplicates for.
	 */
	private void resolveAllDuplicateAuthors(NonViralName name) {
		
		if(duplicateCombinationAuthorTeams.size() > 0){
			name.setCombinationAuthorTeam(duplicateCombinationAuthorTeams.iterator().next());
			ReferenceBase reference = (ReferenceBase) name.getNomenclaturalReference();
			if(reference != null){
				reference.setAuthorTeam(duplicateCombinationAuthorTeams.iterator().next());
			}
			// FIXME TODO resolve multiple duplications. We use first match for a start.
		}
		
		if(duplicateExCombinationAuthorTeams.size() > 0){
			name.setExCombinationAuthorTeam(duplicateExCombinationAuthorTeams.iterator().next());
			// FIXME TODO resolve multiple duplications. We use first match for a start.
		}
		
		if(duplicateBasionymAuthorTeams.size() > 0){
			name.setBasionymAuthorTeam(duplicateBasionymAuthorTeams.iterator().next());
			// FIXME TODO resolve multiple duplications. We use first match for a start.
		}
		
		if(duplicateExBasionymAuthorTeams.size() > 0){
			name.setExBasionymAuthorTeam(duplicateExBasionymAuthorTeams.iterator().next());
			// FIXME TODO resolve multiple duplications. We use first match for a start.
		}
	}
	
	/** 
	 * Splits a NonViralName into its parts and calls methods to find matches for these
	 * parts in the database.
	 * 
	 * @param name	The NonViralName to find matches for.
	 */
	private void findMatches(NonViralName name){
		
		duplicateNames = findMatchingLatinNames(name);
		
		duplicateCombinationAuthorTeams = findMatchingAuthors((TeamOrPersonBase) name.getCombinationAuthorTeam());
		duplicateExCombinationAuthorTeams = findMatchingAuthors((TeamOrPersonBase) name.getExCombinationAuthorTeam());
		duplicateBasionymAuthorTeams = findMatchingAuthors((TeamOrPersonBase) name.getBasionymAuthorTeam());
		duplicateExBasionymAuthorTeams = findMatchingAuthors((TeamOrPersonBase) name.getExBasionymAuthorTeam());
		
		INomenclaturalReference nomenclaturalReference = name.getNomenclaturalReference();
		
		// check if the reference has an inreference and also check if the inReference already exists 
		if(nomenclaturalReference != null){
			ReferenceBase inReference = ((ReferenceBase)nomenclaturalReference).getInReference();
			if(inReference != null){
				doResolveInReferences = true;
				duplicateInReferences = findMatchingNomenclaturalReference(inReference);
			}
		}
		
		duplicateReferences = findMatchingNomenclaturalReference(nomenclaturalReference);
	}
	
	/**
	 * @param nomenclaturalReference	The NomenclaturalReference to find matches for.
	 * @return	A <code>List</code> of possibly matching NomenclaturalReference's.
	 */
	private List<INomenclaturalReference> findMatchingNomenclaturalReference(INomenclaturalReference nomenclaturalReference) {
		if(nomenclaturalReference == null) return new ArrayList<INomenclaturalReference>();
		try{
			return CdmStore.getCommonService().findMatching(nomenclaturalReference, MatchStrategyConfigurator.ReferenceMatchStrategy());
		}catch (MatchException e) {
			logger.error("Error finding matching references", e);
		}
		return null;
	}

	/**
	 * @param authorTeam	The TeamOrPersonBase to find matches for.
	 * @return	A <code>List</code> of possibly matching TeamOrPersonBase's.
	 */
	private List<TeamOrPersonBase> findMatchingAuthors(TeamOrPersonBase authorTeam) {

		if(authorTeam == null){
			return new ArrayList<TeamOrPersonBase>();
		}
		
		try{
			return CdmStore.getCommonService().findMatching(authorTeam, MatchStrategyConfigurator.TeamOrPersonMatchStrategy());
		}catch (MatchException e) {
			logger.error("Error finding matching authors", e);
		}
		return null;
	}

	/**
	 * @param taxonNameBase	The TaxonNameBase to find matches for.
	 * @return	A <code>List</code> of possibly matching TaxonNameBase's.
	 */
	private List<TaxonNameBase> findMatchingLatinNames(TaxonNameBase taxonNameBase) {

		try {
			return CdmStore.getCommonService().findMatching(taxonNameBase, MatchStrategyConfigurator.NonViralNameMatchStrategy());
			
		} catch (MatchException e) {
			logger.error("Error finding matching names", e);
		}
		return null;
	}

	/**
	 * @return The Control this ParseHandler is attached to.
	 */
	public Control getTextWidget() {
		return textWidget;
	}

	/**
	 * @param textWidget The textWidget to set
	 */
	public void setTextWidget(Control textWidget) {
		this.textWidget = textWidget;
		checkControlHasText();
	}
}
