// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference.matching;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.match.MatchException;
import eu.etaxonomy.cdm.strategy.match.MatchMode;
import eu.etaxonomy.taxeditor.parser.MatchStrategyConfigurator;

/**
 * @author n.hoffmann
 * @created Jan 22, 2010
 * @version 1.0
 */
public abstract class AbstractMatchingPreferences<T extends IdentifiableEntity> extends PreferencePage implements
		IWorkbenchPreferencePage {
	private static final Logger logger = Logger
			.getLogger(AbstractMatchingPreferences.class);

	/**
	 * Fields that will be excluded from the display
	 */
	private static final String ExcludePattern = "serialVersionUID|logger|allFields|ajc.*|id|updated|updatedBy|created|createdBy|uuid" +
				"|parsingProblem|problemStarts|problemEnds|PROTECTED|NOT_PROTECTED|propertyChangeSupport";

	protected IMatchStrategy matchStrategy;
	
	protected Class<T> clazz;

	private List<MatchMode> matchModeList;
	
	protected Map<String, Combo> matchModeCombos = new HashMap<String, Combo>();
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	protected Control createContents(Composite parent) {
		final Composite composite = new Composite(parent, SWT.NULL);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		composite.setLayout(gridLayout);
		
		
		for(String fieldName : getFieldNames()){
			createFieldWidget(composite, fieldName);
		}
		
		return composite;
	}
	
	/**
	 * Creates a widget for a field consisting of the label and a combo
	 * 
	 * @see {@link #createMatchModeCombo(Composite, String, MatchMode)}
	 * @param composite
	 * @param fieldName
	 */
	private void createFieldWidget(Composite parent, String fieldName) {
		CLabel label = new CLabel(parent, SWT.NONE);
		label.setText(fieldName);
		
		MatchMode matchMode = matchStrategy.getMatchMode(fieldName);
		
		createMatchModeCombo(parent, fieldName, matchMode);		
	}	
	
	/**
	 * Creates a combo for a field with the currently selected match mode for that field preselected
	 * 
	 * @param parent
	 * @param matchMode
	 */
	private void createMatchModeCombo(Composite parent, String fieldName, MatchMode selectedMatchMode) {
		Combo matchModeCombo = new Combo(parent, SWT.NULL);
		
		for (MatchMode matchMode : getMatchModeList()) {
			matchModeCombo.add(matchMode.name());
		}
		
		int index = getMatchModeList().indexOf(selectedMatchMode);
		
		matchModeCombo.select(index);
		
		matchModeCombo.addSelectionListener(new MatchModeComboSelectionListener(matchModeCombo, fieldName));
		
		matchModeCombos.put(fieldName, matchModeCombo);
	}
	
	/**
	 * This listener updates the cache strategy when a value was changed in one of the combos
	 * 
	 * @author n.hoffmann
	 * @created Jan 28, 2010
	 * @version 1.0
	 */
	private class MatchModeComboSelectionListener extends SelectionAdapter{
		
		private Combo matchModeCombo;
		private String fieldName;

		MatchModeComboSelectionListener(Combo matchModeCombo, String fieldName){
			this.matchModeCombo = matchModeCombo;
			this.fieldName = fieldName;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			MatchMode matchMode = getMatchModeList().get(matchModeCombo.getSelectionIndex());
			try {
				matchStrategy.setMatchMode(fieldName, matchMode);
			} catch (MatchException exception) {
				logger.error(e);
				throw new RuntimeException(exception);
			}
		}
	}

	/**
	 * Transforms the MatchMode enum into a list.
	 * 
	 * @return
	 */
	private List<MatchMode> getMatchModeList(){
		if(matchModeList == null){
			matchModeList = Arrays.asList(MatchMode.values());
		}
		return matchModeList;
	}

	/**
	 * Get names of all declared fields
	 * 
	 * @return
	 */
	private List<String> getFieldNames(){				
		List<Field> fields = new ArrayList<Field>();
		
		fields = getAllFields(fields, clazz);
		List<String> fieldNames = new ArrayList<String>();
		
		for(Field field : fields){
			String fieldName = field.getName();
			if(! fieldName.matches(ExcludePattern)){
				fieldNames.add(fieldName);
			}
		}
		
		return fieldNames;
	}
	
	/**
	 * Get all declared fields including fields of the superclasses.
	 * 
	 * @param fields
	 * @param type
	 * @return
	 */
	public static List<Field> getAllFields(List<Field> fields, Class<?> type) {
        fields.addAll(Arrays.asList(type.getDeclaredFields()));

	    if (type.getSuperclass() != null) {
	        fields = getAllFields(fields, type.getSuperclass());
	    }

	    return fields;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performApply()
	 */
	@Override
	protected void performApply() {
		MatchStrategyConfigurator.setMatchStrategy(matchStrategy);
		super.performApply();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.preference.PreferencePage#performDefaults()
	 */
	@Override
	protected void performDefaults() {
		try {
			// set match strategy to default
			matchStrategy = getDefaultMatchStrategy();
			
			// set combos to their default values
			for(String fieldName : matchModeCombos.keySet()){
				Combo combo = matchModeCombos.get(fieldName);
				MatchMode matchMode = matchStrategy.getMatchMode(fieldName);
				combo.select(matchModeList.indexOf(matchMode));
			}
			
		} catch (MatchException e) {
			logger.error(e);
			throw new RuntimeException(e);
		}
		super.performDefaults();
	}
	
	/**
	 * Returns the default match strategy for the respective class
	 * 
	 * @return
	 * @throws MatchException
	 */
	protected abstract IMatchStrategy getDefaultMatchStrategy() throws MatchException;
}
