/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.store;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.widgets.Display;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.security.authentication.ProviderManager;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.service.IAgentService;
import eu.etaxonomy.cdm.api.service.ICommonService;
import eu.etaxonomy.cdm.api.service.ILocationService;
import eu.etaxonomy.cdm.api.service.IMediaService;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.IReferenceService;
import eu.etaxonomy.cdm.api.service.ITaxonNodeService;
import eu.etaxonomy.cdm.api.service.ITaxonService;
import eu.etaxonomy.cdm.api.service.ITaxonTreeService;
import eu.etaxonomy.cdm.api.service.ITermService;
import eu.etaxonomy.cdm.api.service.IUserService;
import eu.etaxonomy.cdm.api.service.IVocabularyService;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.database.DataSourceNotFoundException;
import eu.etaxonomy.cdm.database.DatabaseSchemaMismatchException;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.DbSchemaValidation;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.cdm.ext.IEditGeoService;
import eu.etaxonomy.cdm.model.agent.AgentBase;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.datasource.CdmDataSourceRepository;
import eu.etaxonomy.taxeditor.datasource.view.CdmDataSourceView;
import eu.etaxonomy.taxeditor.dialogs.LoginDialog;
import eu.etaxonomy.taxeditor.io.ExportHandler;
import eu.etaxonomy.taxeditor.io.ImportHandler;
import eu.etaxonomy.taxeditor.model.IContextListener;
import eu.etaxonomy.taxeditor.model.IContextListener.EventType;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * This implementation of ICdmDataRepository depends on hibernate sessions to store the data correctly 
 * for the current session. No state is held in this class.
 * 
 * Only methods that either get or manipulate data are exposed here. So this class acts as a facade 
 * for the methods in cdmlib-service. 
 * 
 * 
 * @author n.hoffmann
 * @created 17.03.2009
 * @version 1.0
 */
public class CdmStore{
	private static final Logger logger = Logger.getLogger(CdmStore.class);
	
	// FIXME change this to ClassPathResources as soon as it is included into the plugin
	private static final Resource DEFAULT_APPLICATION_CONTEXT = new ClassPathResource("/eu/etaxonomy/cdm/editorApplicationContext.xml", TaxeditorStorePlugin.class.getClassLoader());
	private static final DbSchemaValidation DEFAULT_DB_SCHEMA_VALIDATION = DbSchemaValidation.UPDATE;
	
	private static CdmStore instance;
		
	private CdmApplicationController applicationController;

	private static LoginManager loginManager;
	
	private static ImportHandler importHandler;
	
	private static ExportHandler exportHandler;
	
	private Language language;
	
	private static ListenerList contextListeners = new ListenerList();
	
	private static ICdmDataSource cdmDatasource;
	
	private boolean isConnected;
	
	
	/**
	 * 
	 * @param applicationContextBean
	 * @return
	 */
	protected static CdmStore getDefault(){
		if(instance != null && instance.isConnected){
			return instance;
		}else if(instance == null){
			
			initialize();
			
			return instance;
		}else if(!instance.isConnected){
			
			StoreUtil.warningDialog("No datasource connection", "Application is not connected to a datastore.");
			
			// TODO open datasource view
			
		}	
		return null;
	}
	
	/**
	 * Initialize the with the last edited datasource
	 */
	public static void initialize() {
		
		ICdmDataSource datasource = CdmDataSourceRepository.getCurrentDataSource();
		
		initialize(datasource);
	}
	
	/**
	 * Initialize with a specific datasource
	 * 
	 * @param datasource
	 */
	public static void initialize(ICdmDataSource datasource) {
		initialize(datasource, DEFAULT_DB_SCHEMA_VALIDATION, DEFAULT_APPLICATION_CONTEXT);
	}
	
	/**
	 * Initialize and provide
	 * 
	 * @param datasource
	 * @param dbSchemaValidation
	 * @param applicationContextBean
	 */
	private static void initialize(final ICdmDataSource datasource, final DbSchemaValidation dbSchemaValidation, final Resource applicationContextBean){	
		if(isActive()){
//			close();
			logger.error("Application context already initialized.");
			return;
		}
		
		logger.info("Initializing application context ...");
		
		try {
			if (datasource.testConnection()) {
				cdmDatasource = datasource;
				
				IRunnableWithProgress runnable = new IRunnableWithProgress() {
					
					public void run(IProgressMonitor monitor) throws InvocationTargetException,
							InterruptedException {
						String message = "";
						if(cdmDatasource.getDatabaseType().equals(DatabaseTypeEnum.H2)){
							message = " local CDM Store ";
						}else{
							message = " CDM Community Store ";
						}
						message += "'" + cdmDatasource.getName() + "'";
						
						monitor.beginTask("Establishing connection to" + message + ". \nThis might take  while.", 10);
						// TODO get real feedback from CDMStore initialisation process
						monitor.worked(3);
						
						instance = new CdmStore(datasource, dbSchemaValidation, applicationContextBean);
						
						monitor.done();
					}
				};
				
				StoreUtil.run(false, false, runnable);
				
				if (!getCommonService().isDatabaseSchemaCompatible()) {
					close();
					// Show the datasource chooser
					if(CdmDataSourceRepository.getAll().size() > 1){
						// Show an error message
						StoreUtil.errorDialog("DatabaseCompatibilityCheck failed", "The database schema for the chosen " +
								"datasource '" + datasource + "' \n is not valid for this version of the taxonomic editor. \n" +
								"Please choose a new data source to connect to in the Datasource View.");
						
						// Show datasource view if not shown yet
						StoreUtil.showView(CdmDataSourceView.ID);
					} else {
						throw new DatabaseSchemaMismatchException();
					}
				} else {
					notifyContextStart();
				
					logger.info("Application context initialized.");
				}				
			}			
		} catch (DataSourceNotFoundException e) {
			StoreUtil.errorDialog("Chosen Datasource is not available", "Could not connect to the chosen " +
					"datasource '" + datasource + "'. Please check settings in datasources.xml. If the datasource " +
							"is located on a remote machine, make sure you are connected to the network.");
			logger.error(e);
		} catch (NullPointerException e){
			StoreUtil.errorDialog("Corrupt Datasource", "The configuration for the chosen " +
					"datasource '" + datasource + "' is corrupt. Please check settings in datasources.xml.");
			logger.error(e);
		} catch (DatabaseSchemaMismatchException e){
			StoreUtil.errorDialog("Database Schema incompatible", "The database schema for the chosen " +
					"datasource '" + datasource + "' is not valid for this version of the taxonomic editor.");
			logger.error(e);
		} catch (Exception e){
			StoreUtil.errorDialog("Unknown Datasource Error", "An error occurred while connecting to the datasource." +
					"Please refer to the error log.");
			logger.error(e);
		} 
	}

	/**
	 * Closes the current application context
	 */
	public static void close(){
		notifyContextAboutToStop();
		if(isActive() && StoreUtil.closeAll()){
			notifyContextStop();
			instance.getApplicationController().close();
			instance.isConnected = false;
			cdmDatasource = null;
			
		}
	}

	/**
	 * 
	 */
	private CdmStore(ICdmDataSource dataSource, DbSchemaValidation dbSchemaValidation, Resource applicationContextBean) {
		try {
			// this should be more modulized 
			applicationController = CdmApplicationController.NewInstance(applicationContextBean, dataSource, dbSchemaValidation, false);
			// 
			isConnected = true;		
			
			cdmDatasource = dataSource;
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}
	
	/**
	 * All calls to the datastore require 
	 * 
	 * @return
	 */
	private CdmApplicationController getApplicationController(){
		try{
			return applicationController;
		}catch(Exception e){
			logger.error("Exception thrown", e);
		}
		return null;
	}

	/*
	 * CONVERSATIONS
	 */

	/**
	 * Creates a new conversation, binds resources to the conversation and 
	 * start a transaction for this conversation.
	 * 
	 * @return
	 */
	public static ConversationHolder createConversation() {
		ConversationHolder conversation = getDefault().getApplicationController().NewConversation();
		
		conversation.startTransaction();
		return conversation;
	}
	
	/*
	 * EXPOSING SERVICES
	 */
	
	public static ITaxonService getTaxonService(){ return getDefault().getApplicationController().getTaxonService();}
	
	public static ITaxonTreeService getTaxonTreeService() { return getDefault().getApplicationController().getTaxonTreeService();}
	
	public static ITaxonNodeService getTaxonNodeService() { return getDefault().getApplicationController().getTaxonNodeService();}
	
	public static INameService getNameService(){ return getDefault().getApplicationController().getNameService();}
	
	public static IReferenceService getReferenceService(){ return getDefault().getApplicationController().getReferenceService();}
	
	public static ILocationService getLocationService(){ return getDefault().getApplicationController().getLocationService();}

	public static ProviderManager getAuthenticationManager() { return getDefault().getApplicationController().getAuthenticationManager();}

	public static IUserService getUserService() { return getDefault().getApplicationController().getUserService(); }
	
	public static ICommonService getCommonService() { return getDefault().getApplicationController().getCommonService(); }
	
	public static IAgentService getAgentService() { return getDefault().getApplicationController().getAgentService(); }
	
	public static ITermService getTermService() { return getDefault().getApplicationController().getTermService(); }
	
	public static IVocabularyService getVocabularyService() { return getDefault().getApplicationController().getVocabularyService(); }
	
	public static IMediaService getMediaService() { return getDefault().getApplicationController().getMediaService(); }
	
	public static IOccurrenceService getOccurrenceService() { return getDefault().getApplicationController().getOccurrenceService(); }
	
	public static IEditGeoService getGeoService(){
		return (IEditGeoService) getDefault().getApplicationController().getBean("editGeoService");
	}
		
	/*
	 * METHODS TO FIND ENTITIES
	 */
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<TaxonNameBase> findNames(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getNameService().getNamesByName(titleSearchString);
	}

	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<IdentifiableEntity> findTaxaAndNames(ITaxonServiceConfigurator configurator){
		return getTaxonService().findTaxaAndNames(configurator).getRecords();
	}
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<ReferenceBase> findReferences(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getReferenceService().findByTitle(null, titleSearchString, null, null, null, null, null, null).getRecords();
	}
	
	/**
	 * 
	 * @param configurator
	 * @return
	 */
	public static List<AgentBase> findAgents(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getAgentService().findByTitle(null, titleSearchString, null, null, null, null, null, null).getRecords();
	}

	public static List<TeamOrPersonBase> findTeamOrPersons(IIdentifiableEntityServiceConfigurator configurator){
		// TODO move this to cdmlib
		List<TeamOrPersonBase> result = new ArrayList<TeamOrPersonBase>();
		for (AgentBase agent : findAgents(configurator)) {
			if (agent instanceof TeamOrPersonBase) {
				result.add((TeamOrPersonBase) agent);
			}
		}
		return result;
	}
	
	public static List<SpecimenOrObservationBase> findOccurrences(IIdentifiableEntityServiceConfigurator configurator){
		// TODO we want to use IIdentifiableEntityServiceConfigurator for all find methods
		// unfortunately this is not consistently implemented in the library.
		// FIXME use proper method once it is implemented in the library
		String titleSearchString = configurator.getTitleSearchString().replace("*", "%");
		
		return getOccurrenceService().findByTitle(SpecimenOrObservationBase.class, titleSearchString, null, null, null, null, null, null).getRecords();
	}
	
	/*
	 * LANGUAGE
	 */
	
	/** 
	 * @return
	 */
	public static Language getDefaultLanguage(){
		if(getDefault().getLanguage() == null){
			getDefault().setLanguage(Language.DEFAULT());
		}
		return getDefault().getLanguage();
	}

	public static void setDefaultLanguage(Language language){
		getDefault().setLanguage(language);
	}
	
	/**
	 * @return the language
	 */
	private Language getLanguage() {
		return language;
	}

	/**
	 * @param language the language to set
	 */
	private void setLanguage(Language language) {
		this.language = language;
	}
	
	/*
	 * LOGIN
	 */
	
	/**
	 * 
	 */
	public static LoginManager getLoginManager(){
		if(loginManager == null){
			loginManager = new LoginManager();
		}
		return loginManager;
	}
	
	private int authenticate(){
		
		LoginDialog loginDialog = new LoginDialog(StoreUtil.getShell());
		return loginDialog.open();
	}
	
	/*
	 * IMPORT/EXPORT FACTORIES
	 */
	
	/**
	 * 
	 */
	public static synchronized ImportHandler getImportHandler(){
		if(importHandler == null){
			importHandler = ImportHandler.NewInstance(getDefault().getApplicationController());
		}
		return importHandler;
	}
	
	/**
	 * 
	 * @return
	 */
	public static synchronized ExportHandler getExportHandler(){
		if(exportHandler == null){
			exportHandler = ExportHandler.NewInstance(getDefault().getApplicationController());
		}
		return exportHandler;
	}
	
	/**
	 * Whether this CdmStore is currently connected to a datasource
	 * @return
	 */
	public static boolean isActive(){
		return instance != null && instance.isConnected;
	}

	public static void addContextListener(IContextListener listener){
		contextListeners.add(listener);
	}
	
	public static void removeContextListener(IContextListener listener) {
		contextListeners.remove(listener);
	}
	
	public static void notifyContextStart() {
		logger.warn("Application event occured");
		
		instance.authenticate();
		
		
		for(Object listener : contextListeners.getListeners()){
			((IContextListener) listener).onContextEvent(EventType.START);
		}		
	}
	
	public static void notifyContextAboutToStop(){
		for(Object listener : contextListeners.getListeners()){
			((IContextListener) listener).onContextEvent(EventType.ABOUT_TO_STOP);
		}	
	}
	
	public static void notifyContextStop() {
		logger.warn("Application event occured");
		
		for(Object listener : contextListeners.getListeners()){
			((IContextListener) listener).onContextEvent(EventType.STOP);
		}		
	}

	public static ICdmDataSource getDataSource(){
		if(isActive()){
			return cdmDatasource;
		}
		return null;
	}
	
	public static void createDefaultClassification(ConversationHolder conversation){
		TaxonomicTree defaultClassification = TaxonomicTree.NewInstance("My Classification");
		getTaxonTreeService().saveOrUpdate(defaultClassification);
		conversation.commit(true);
	}

}
